@@ -5,12 +5,14 @@
 # Please see the file LICENSE.txt for details.
 
 import math
+import os.path
 import itertools
 import numpy as np
 
 from ginga.misc import Bunch, Settings, Callback
-from ginga import AutoCuts, trcalc
+from ginga import trcalc
 from ginga import cmap, imap
+from ginga.util.paths import icondir
 
 
 class ImageViewBindings(object):
@@ -26,7 +28,6 @@ class ImageViewBindings(object):
 
         self.canpan = False
         self.canzoom = False
-        self._ispanning = False
         self.cancut = False
         self.cancmap = False
         self.canflip = False
@@ -50,21 +51,25 @@ class ImageViewBindings(object):
                                              logger=self.logger)
             self.initialize_settings(settings)
         self.settings = settings
-        self.autocuts = AutoCuts.ZScale(self.logger)
 
         self.features = dict(
             # name, attr pairs
             pan='canpan', zoom='canzoom', cuts='cancut', cmap='cancmap',
             flip='canflip', rotate='canrotate')
+        self.cursor_map = {}
 
     def initialize_settings(self, settings):
         settings.addSettings(
             # You should rarely have to change these.
             btn_nobtn = 0x0,
             btn_left  = 0x1,
-            btn_middle= 0x2,
+            btn_middle = 0x2,
             btn_right = 0x4,
 
+            # define our cursors
+            ## cur_pick = 'thinCrossCursor',
+            ## cur_pan = 'openHandCursor',
+
             # Set up our standard modifiers
             mod_shift = ['shift_l', 'shift_r'],
             mod_ctrl = ['control_l', 'control_r'],
@@ -77,8 +82,10 @@ class ImageViewBindings(object):
             dmod_dist = ['d', None, None],
             dmod_contrast = ['t', None, None],
             dmod_rotate = ['r', None, None],
-            dmod_pan = ['q', None, None],
-            dmod_freepan = ['w', None, None],
+            dmod_pan = ['q', None, 'pan'],
+            dmod_freepan = ['w', None, 'pan'],
+            dmod_camera = [None, None, 'pan'],
+            dmod_naxis = [None, None, None],
 
             default_mode_type = 'oneshot',
             default_lock_mode_type = 'softlock',
@@ -101,6 +108,11 @@ class ImageViewBindings(object):
             kp_pan_right = ['pan+*+right', 'freepan+*+right'],
             kp_pan_up = ['pan+*+up', 'freepan+*+up'],
             kp_pan_down = ['pan+*+down', 'freepan+*+down'],
+            kp_pan_px_xminus = ['shift+left'],
+            kp_pan_px_xplus = ['shift+right'],
+            kp_pan_px_yminus = ['shift+down'],
+            kp_pan_px_yplus = ['shift+up'],
+            kp_pan_px_center = ['shift+home'],
             kp_center = ['c', 'pan+c', 'freepan+c'],
             kp_cut_255 = ['cuts+A'],
             kp_cut_minmax = ['cuts+S'],
@@ -135,9 +147,14 @@ class ImageViewBindings(object):
             kp_reset = ['escape'],
             kp_lock = ['L'],
             kp_softlock = ['l'],
+            kp_camera_save = ['camera+s'],
+            kp_camera_reset = ['camera+r'],
+            kp_camera_toggle3d = ['camera+3'],
 
             # pct of a window of data to move with pan key commands
             key_pan_pct = 0.666667,
+            # amount to move (in pixels) when using key pan arrow
+            key_pan_px_delta = 1.0,
 
             # SCROLLING/WHEEL
             sc_pan = ['ctrl+scroll'],
@@ -153,7 +170,8 @@ class ImageViewBindings(object):
             sc_dist = ['dist+scroll'],
             sc_cmap = ['cmap+scroll'],
             sc_imap = ['cmap+ctrl+scroll'],
-            #sc_draw = ['draw+scroll'],
+            sc_camera_track = ['camera+scroll'],
+            sc_naxis = ['naxis+scroll'],
 
             scroll_pan_acceleration = 1.0,
             # 1.0 is appropriate for a mouse, 0.1 for most trackpads
@@ -165,6 +183,8 @@ class ImageViewBindings(object):
             mouse_rotate_acceleration = 0.75,
             pan_reverse = False,
             pan_multiplier = 1.0,
+            pan_min_scroll_thumb_pct = 0.0,
+            pan_max_scroll_thumb_pct = 0.8,
             zoom_scroll_reverse = False,
 
             # MOUSE/BUTTON
@@ -189,6 +209,9 @@ class ImageViewBindings(object):
             ms_panset = ['pan+middle', 'shift+left', 'middle'],
             ms_cmap_rotate = ['cmap+left'],
             ms_cmap_restore = ['cmap+right'],
+            ms_camera_orbit = ['camera+left'],
+            ms_camera_pan_delta = ['camera+right'],
+            ms_naxis = ['naxis+left'],
 
             # GESTURES (some backends only)
             gs_pinch = [],
@@ -219,6 +242,8 @@ class ImageViewBindings(object):
         bindmap.clear_button_map()
         bindmap.clear_event_map()
 
+        bindmap.add_callback('mode-set', self.mode_set_cb, viewer)
+
         # Set up bindings
         self.setup_settings_events(viewer, bindmap)
 
@@ -226,6 +251,10 @@ class ImageViewBindings(object):
         bindmap = viewer.get_bindmap()
         bindmap.set_mode(name, mode_type=mode_type)
 
+    def mode_set_cb(self, bm, mode, mode_type, viewer):
+        cursor_name = self.cursor_map.get(mode, 'pick')
+        viewer.switch_cursor(cursor_name)
+
     def parse_combo(self, combo, modes_set, modifiers_set, pfx):
         """
         Parse a string into a mode, a set of modifiers and a trigger.
@@ -284,15 +313,21 @@ class ImageViewBindings(object):
                     keyname = combo
                     bindmap.add_modifier(keyname, modname)
 
+            elif name.startswith('cur_'):
+                curname = name[4:]
+                self.add_cursor(viewer, curname, value)
+
             elif name.startswith('btn_'):
                 btnname = name[4:]
                 bindmap.map_button(value, btnname)
 
             elif name.startswith('dmod_'):
                 mode_name = name[5:]
-                keyname, mode_type, msg = value
+                keyname, mode_type, curname = value
                 bindmap.add_mode(keyname, mode_name, mode_type=mode_type,
-                                     msg=msg)
+                                     msg=None)
+                if curname is not None:
+                    self.cursor_map[mode_name] = curname
 
         modes_set = bindmap.get_modes()
         modifiers_set = bindmap.get_modifiers()
@@ -357,9 +392,14 @@ class ImageViewBindings(object):
     def reset(self, viewer):
         bindmap = viewer.get_bindmap()
         bindmap.reset_mode(viewer)
-        self.pan_stop(viewer)
         viewer.onscreen_message(None)
 
+    def add_cursor(self, viewer, curname, curpath):
+        if not curpath.startswith('/'):
+            curpath = os.path.join(icondir, curpath)
+        cursor = viewer.make_cursor(curpath, 8, 8)
+        viewer.define_cursor(curname, cursor)
+
     #####  ENABLERS #####
     # These methods are a quick way to enable or disable certain user
     # interface features in a ImageView window
@@ -513,9 +553,6 @@ class ImageViewBindings(object):
         qx1, qx2 = np.min(x), np.max(x)
         qy1, qy2 = np.min(y), np.max(y)
 
-        qx1, qx2 = max(rx1, qx1), min(rx2, qx2)
-        qy1, qy2 = max(ry1, qy1), min(ry2, qy2)
-
         # this is the range of X and Y of the entire image
         # in the viewer (unscaled)
         rng_x, rng_y = abs(rx2 - rx1), abs(ry2 - ry1)
@@ -524,20 +561,27 @@ class ImageViewBindings(object):
         abs_x, abs_y = abs(qx2 - qx1), abs(qy2 - qy1)
 
         # calculate the width of the slider arms as a ratio
-        xthm_pct = max(0.0, min(abs_x / (rx2 - rx1), 1.0))
-        ythm_pct = max(0.0, min(abs_y / (ry2 - ry1), 1.0))
+        min_pct = self.settings.get('pan_min_scroll_thumb_pct', 0.0)
+        max_pct = self.settings.get('pan_max_scroll_thumb_pct', 0.8)
+        xthm_pct = max(min_pct, min(abs_x / (rx2 - rx1), max_pct))
+        ythm_pct = max(min_pct, min(abs_y / (ry2 - ry1), max_pct))
 
         # calculate the pan position as a ratio
         pct_x = min(max(0.0, abs(0.0 - rx1) / rng_x), 1.0)
         pct_y = min(max(0.0, abs(0.0 - ry1) / rng_y), 1.0)
 
-        return Bunch.Bunch(rng_x=rng_x, rng_y=rng_y, vis_x=abs_x, vis_y=abs_y,
+        bnch = Bunch.Bunch(rng_x=rng_x, rng_y=rng_y, vis_x=abs_x, vis_y=abs_y,
                            thm_pct_x=xthm_pct, thm_pct_y=ythm_pct,
                            pan_pct_x=pct_x, pan_pct_y=pct_y)
+        return bnch
 
     def pan_by_pct(self, viewer, pct_x, pct_y, pad=0):
         """Called when the scroll bars are adjusted by the user.
         """
+        # Sanity check on inputs
+        pct_x = min(1.0, max(0.0, pct_x))
+        pct_y = min(1.0, max(0.0, pct_y))
+
         limits = viewer.get_limits()
 
         tr = viewer.tform['data_to_scrollbar']
@@ -608,6 +652,19 @@ class ImageViewBindings(object):
         # update the pan position by pct
         self.pan_by_pct(viewer, res.pan_pct_x, pct_y)
 
+    def pan_delta_px(self, viewer, x_px, y_px):
+        pan_x, pan_y = viewer.get_pan(coord='data')[:2]
+        pan_x += x_px
+        pan_y += y_px
+        viewer.panset_xy(pan_x, pan_y)
+
+    def pan_center_px(self, viewer):
+        data_x, data_y = viewer.get_last_data_xy()
+        ndata_x = float(int(data_x + viewer.data_off))
+        ndata_y = float(int(data_y + viewer.data_off))
+        x_px, y_px = ndata_x - data_x, ndata_y - data_y
+        self.pan_delta_px(viewer, x_px, y_px)
+
     def get_direction(self, direction, rev=False):
         """
         Translate a direction in compass degrees into 'up' or 'down'.
@@ -625,6 +682,15 @@ class ImageViewBindings(object):
         else:
             return 'none'
 
+    def get_win_xy(self, viewer):
+        x, y = viewer.get_last_win_xy()
+
+        if not viewer.window_has_origin_upper():
+            wd, ht = viewer.get_window_size()
+            y = ht - y
+
+        return x, y
+
     def _tweak_colormap(self, viewer, x, y, mode):
         win_wd, win_ht = viewer.get_window_size()
 
@@ -750,7 +816,9 @@ class ImageViewBindings(object):
         elif direction == 'down':
             mult = 1.0 / factor
         scale_x, scale_y = scale_x * mult, scale_y * mult
-        viewer.scale_to(scale_x, scale_y)
+        scale = max(scale_x, scale_y)
+        #viewer.scale_to(scale_x, scale_y)
+        viewer.scale_to(scale, scale)
         if msg:
             viewer.onscreen_message(viewer.get_scale_text(),
                                        delay=0.4)
@@ -762,7 +830,6 @@ class ImageViewBindings(object):
         direction = 0.0
         if delta < 0.0:
             direction = 180.0
-        #print("factor=%f direction=%f" % (factor, direction))
         self._start_x = x
         self._scale_image(viewer, direction, factor, msg=msg)
 
@@ -890,14 +957,9 @@ class ImageViewBindings(object):
     def _rotate_xy(self, viewer, x, y, msg=True):
         msg = self.settings.get('msg_rotate', msg)
         ctr_x, ctr_y = viewer.get_center()
-        if not viewer.window_has_origin_upper():
-            deg1 = math.degrees(math.atan2(self._start_y - ctr_y,
-                                           self._start_x - ctr_x))
-            deg2 = math.degrees(math.atan2(y - ctr_y, x - ctr_x))
-        else:
-            deg1 = math.degrees(math.atan2(ctr_y - self._start_y,
-                                           self._start_x - ctr_x))
-            deg2 = math.degrees(math.atan2(ctr_y - y, x - ctr_x))
+        deg1 = math.degrees(math.atan2(ctr_y - self._start_y,
+                                       self._start_x - ctr_x))
+        deg2 = math.degrees(math.atan2(ctr_y - y, x - ctr_x))
         delta_deg = deg2 - deg1
         deg = math.fmod(self._start_rot + delta_deg, 360.0)
         if msg:
@@ -941,26 +1003,18 @@ class ImageViewBindings(object):
                 degn, str(xflip)), delay=1.0)
 
     def to_default_mode(self, viewer):
-        self._ispanning = False
-        viewer.switch_cursor('pick')
+        self.reset(viewer)
 
     def pan_start(self, viewer, ptype=1):
-        # If already panning then ignore multiple keystrokes
-        if self._ispanning:
-            return
         self._pantype = ptype
-        viewer.switch_cursor('pan')
-        self._ispanning = True
 
     def pan_set_origin(self, viewer, win_x, win_y, data_x, data_y):
         self._start_x, self._start_y = viewer.window_to_offset(win_x, win_y)
         self._start_panx, self._start_pany = viewer.get_pan()
 
     def pan_stop(self, viewer):
-        self._ispanning = False
         self._start_x = None
         self._pantype = 1
-        self.to_default_mode(viewer)
 
     def restore_contrast(self, viewer, msg=True):
         msg = self.settings.get('msg_cmap', msg)
@@ -1036,6 +1090,42 @@ class ImageViewBindings(object):
         self.pan_ud(viewer, amt, -1.0, msg=msg)
         return True
 
+    def kp_pan_px_xminus(self, viewer, event, data_x, data_y, msg=True):
+        if not self.canpan:
+            return False
+        px_amt = self.settings.get('key_pan_px_delta', 1.0)
+        self.pan_delta_px(viewer, -px_amt, 0.0)
+        return True
+
+    def kp_pan_px_xplus(self, viewer, event, data_x, data_y, msg=True):
+        if not self.canpan:
+            return False
+        px_amt = self.settings.get('key_pan_px_delta', 1.0)
+        self.pan_delta_px(viewer, px_amt, 0.0)
+        return True
+
+    def kp_pan_px_yminus(self, viewer, event, data_x, data_y, msg=True):
+        if not self.canpan:
+            return False
+        px_amt = self.settings.get('key_pan_px_delta', 1.0)
+        self.pan_delta_px(viewer, 0.0, -px_amt)
+        return True
+
+    def kp_pan_px_yplus(self, viewer, event, data_x, data_y, msg=True):
+        if not self.canpan:
+            return False
+        px_amt = self.settings.get('key_pan_px_delta', 1.0)
+        self.pan_delta_px(viewer, 0.0, px_amt)
+        return True
+
+    def kp_pan_px_center(self, viewer, event, data_x, data_y, msg=True):
+        """This pans so that the cursor is over the center of the
+        current pixel."""
+        if not self.canpan:
+            return False
+        self.pan_center_px(viewer)
+        return True
+
     def kp_center(self, viewer, event, data_x, data_y):
         if self.canpan:
             viewer.center_image()
@@ -1366,7 +1456,8 @@ class ImageViewBindings(object):
             return True
         msg = self.settings.get('msg_zoom', msg)
 
-        x, y = viewer.get_last_win_xy()
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._zoom_xy(viewer, x, y)
 
@@ -1451,13 +1542,14 @@ class ImageViewBindings(object):
             return True
         msg = self.settings.get('msg_rotate', msg)
 
-        x, y = viewer.get_last_win_xy()
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._rotate_xy(viewer, x, y)
 
         elif event.state == 'down':
             if msg:
-                viewer.onscreen_message("Rotate (drag mouse L-R)",
+                viewer.onscreen_message("Rotate (drag around center)",
                                            delay=1.0)
             self._start_x, self._start_y = x, y
             self._start_rot = viewer.get_rotation()
@@ -1486,9 +1578,8 @@ class ImageViewBindings(object):
             return True
         msg = self.settings.get('msg_contrast', msg)
 
-        x, y = viewer.get_last_win_xy()
-        if not viewer.window_has_origin_upper():
-            y = viewer._imgwin_ht - y
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._tweak_colormap(viewer, x, y, 'preview')
 
@@ -1519,9 +1610,8 @@ class ImageViewBindings(object):
             return True
         msg = self.settings.get('msg_cmap', msg)
 
-        x, y = viewer.get_last_win_xy()
-        if not viewer.window_has_origin_upper():
-            y = viewer._imgwin_ht - y
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._rotate_colormap(viewer, x, y, 'preview')
 
@@ -1593,7 +1683,8 @@ class ImageViewBindings(object):
         if not self.cancut:
             return True
 
-        x, y = viewer.get_last_win_xy()
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._cutlow_xy(viewer, x, y)
 
@@ -1611,7 +1702,8 @@ class ImageViewBindings(object):
         if not self.cancut:
             return True
 
-        x, y = viewer.get_last_win_xy()
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._cuthigh_xy(viewer, x, y)
 
@@ -1629,9 +1721,8 @@ class ImageViewBindings(object):
         if not self.cancut:
             return True
 
-        x, y = viewer.get_last_win_xy()
-        if not viewer.window_has_origin_upper():
-            y = viewer._imgwin_ht - y
+        x, y = self.get_win_xy(viewer)
+
         if event.state == 'move':
             self._cutboth_xy(viewer, x, y)
 
@@ -1639,7 +1730,7 @@ class ImageViewBindings(object):
             self._start_x, self._start_y = x, y
             image = viewer.get_image()
             #self._loval, self._hival = viewer.get_cut_levels()
-            self._loval, self._hival = self.autocuts.calc_cut_levels(image)
+            self._loval, self._hival = viewer.autocuts.calc_cut_levels(image)
 
         else:
             viewer.onscreen_message(None)
@@ -1658,6 +1749,33 @@ class ImageViewBindings(object):
             self._panset(viewer, data_x, data_y, msg=msg)
         return True
 
+    def ms_naxis(self, viewer, event, data_x, data_y, msg=True):
+
+        # TODO: be able to pick axis
+        axis = 2
+        x, y = self.get_win_xy(viewer)
+
+        image = viewer.get_image()
+        if image is None:
+            return
+        mddata = image.get_mddata()
+        if len(mddata.shape) < axis+1:
+            # image dimensions < 3D
+            return
+
+        naxispath = list(image.naxispath)
+        axis_lim = mddata.shape[axis]
+        m = axis - 2
+
+        if event.state in ('down', 'move'):
+            win_wd, win_ht = viewer.get_window_size()
+            x_pct = x / float(win_wd)
+            idx = int(x_pct * axis_lim)
+            naxispath[m] = idx
+            image.set_naxispath(naxispath)
+
+        return True
+
     #####  SCROLL ACTION CALLBACKS #####
 
     def sc_cuts_coarse(self, viewer, event, msg=True):
@@ -1687,7 +1805,6 @@ class ImageViewBindings(object):
         return True
 
     def zoom_step(self, viewer, event, msg=True, origin=None, adjust=1.5):
-
         with viewer.suppress_redraw:
 
             if origin is not None:
@@ -1786,6 +1903,7 @@ class ImageViewBindings(object):
         # User has "Pan Reverse" preference set?
         rev = self.settings.get('pan_reverse', False)
         direction = event.direction
+
         if rev:
             direction = math.fmod(direction + 180.0, 360.0)
 
@@ -1806,6 +1924,35 @@ class ImageViewBindings(object):
         event.amount = event.amount / 5.0
         return self.sc_pan(viewer, event, msg=msg)
 
+    def sc_naxis(self, viewer, event, msg=True):
+
+        # TODO: be able to pick axis
+        axis = 2
+        direction = self.get_direction(event.direction)
+
+        image = viewer.get_image()
+        if image is None:
+            return
+        mddata = image.get_mddata()
+        if len(mddata.shape) < axis+1:
+            # image dimensions < 3D
+            return
+
+        naxispath = list(image.naxispath)
+        axis_lim = mddata.shape[axis]
+        m = axis - 2
+
+        idx = naxispath[m]
+        if direction == 'down':
+            idx = (idx + 1) % axis_lim
+        else:
+            idx = idx - 1
+            if idx < 0:
+                idx = axis_lim - 1
+
+        naxispath[m] = idx
+        image.set_naxispath(naxispath)
+
     def sc_dist(self, viewer, event, msg=True):
 
         direction = self.get_direction(event.direction)
@@ -1836,8 +1983,15 @@ class ImageViewBindings(object):
             if self.canzoom and ('zoom' in pinch_actions):
                 scale_accel = self.settings.get('pinch_zoom_acceleration', 1.0)
                 scale = scale * scale_accel
-                scale_x, scale_y = (self._start_scale_x * scale,
-                                    self._start_scale_y * scale)
+                # NOTE: scale reported by Qt (the only toolkit that really gives
+                # us a pinch gesture right now) the scale reports are iterative
+                # so it makes more sense to keep updating the scale than to base
+                # the scale on the original scale captured when the gesture starts
+                ## scale_x, scale_y = (self._start_scale_x * scale,
+                ##                     self._start_scale_y * scale)
+                scale_x, scale_y = viewer.get_scale_xy()
+                scale_x, scale_y = scale_x * scale, scale_y * scale
+
                 viewer.scale_to(scale_x, scale_y)
                 msg_str = viewer.get_scale_text()
                 msg = self.settings.get('msg_zoom', True)
@@ -1894,6 +2048,116 @@ class ImageViewBindings(object):
                 viewer.onscreen_message(msg_str, delay=0.4)
         return True
 
+
+    ##### CAMERA MOTION CALLBACKS #####
+
+    def get_camera(self, viewer):
+        renderer = viewer.renderer
+        if not hasattr(renderer, 'camera'):
+            return None, None
+        return renderer.camera
+
+
+    def sc_camera_track(self, viewer, event, msg=True):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return
+
+        delta = event.amount * 6
+        direction = self.get_direction(event.direction)
+        if direction == 'down':
+            delta = - delta
+
+        camera.track(delta)
+
+        viewer.gl_update()
+        return True
+
+
+    def ms_camera_orbit(self, viewer, event, data_x, data_y, msg=True):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return False
+
+        x, y = self.get_win_xy(viewer)
+
+        if event.state == 'move':
+            camera.orbit(self._start_x, self._start_y, x, y)
+            self._start_x, self._start_y = x, y
+            pos = tuple(camera.position.get())
+            mst = "Camera position: (%.4f, %.4f, %.4f)" % pos
+            ## if msg:
+            ##     viewer.onscreen_message(mst, delay=0.5)
+
+        elif event.state == 'down':
+            self._start_x, self._start_y = x, y
+
+        ## else:
+        ##     viewer.onscreen_message(None)
+
+        viewer.gl_update()
+        return True
+
+
+    def ms_camera_pan_delta(self, viewer, event, data_x, data_y, msg=True):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return False
+
+        x, y = self.get_win_xy(viewer)
+
+        if event.state == 'move':
+            dx, dy = x - self._start_x, self._start_y - y
+            camera.pan_delta(dx, dy)
+            self._start_x, self._start_y = x, y
+
+        elif event.state == 'down':
+            self._start_x, self._start_y = x, y
+            ## if msg:
+            ##     viewer.onscreen_message("Camera translate", delay=1.0)
+
+        ## else:
+        ##     viewer.onscreen_message(None)
+
+        viewer.gl_update()
+        return True
+
+    def kp_camera_reset(self, viewer, event, data_x, data_y):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return False
+
+        camera.reset()
+        viewer.onscreen_message("Reset camera", delay=0.5)
+        viewer.gl_update()
+        return True
+
+    def kp_camera_save(self, viewer, event, data_x, data_y):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return False
+
+        camera.save_positions()
+        viewer.onscreen_message("Saved camera position", delay=0.5)
+        return True
+
+    def kp_camera_toggle3d(self, viewer, event, data_x, data_y):
+        camera = self.get_camera(viewer)
+        if camera is None:
+            # this viewer doesn't have a camera
+            return False
+
+        renderer = viewer.renderer
+        renderer.mode3d = not renderer.mode3d
+        viewer.gl_update()
+        return True
+
+
 class UIEvent(object):
     pass
 
@@ -2011,6 +2275,14 @@ class BindingMapper(Callback.Callbacks):
     def clear_mode_map(self):
         self.mode_map = {}
 
+    def has_mode(self, mode_name):
+        key = 'mode_%s' % mode_name
+        return key in self.mode_map
+
+    def remove_mode(self, mode_name):
+        key = 'mode_%s' % mode_name
+        del self.mode_map[key]
+
     def current_mode(self):
         return (self._kbdmode, self._kbdmode_type)
 
@@ -2024,7 +2296,9 @@ class BindingMapper(Callback.Callbacks):
                 mode_type, self._kbdmode_types))
 
         bnch = Bunch.Bunch(name=mode_name, type=mode_type, msg=msg)
-        self.mode_map[keyname] = bnch
+        if keyname is not None:
+            # No key to launch this mode
+            self.mode_map[keyname] = bnch
         self.mode_map['mode_%s' % mode_name] = bnch
 
     def set_mode(self, name, mode_type=None):
@@ -2105,8 +2379,6 @@ class BindingMapper(Callback.Callbacks):
         if not has_focus:
             # fixes a problem with not receiving key release events when the
             # window loses focus
-            # TODO: does this need to force a key release callback if there
-            # are any keys held?
             self._modifiers = frozenset([])
         return True
 
