@@ -4,7 +4,8 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import time
+
+# import time
 
 from ginga.gtk3w import GtkHelp
 import ginga.util.six as six
@@ -17,15 +18,39 @@ from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import GdkPixbuf
 
+has_webkit = False
+try:
+    # this is necessary to prevent a warning message on import
+    import gi
+    gi.require_version('WebKit', '3.0')
+
+    from gi.repository import WebKit  # noqa
+    has_webkit = True
+except ImportError:
+    pass
+
+__all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
+           'TextArea', 'Label', 'Button', 'ComboBox',
+           'SpinBox', 'Slider', 'ScrollBar', 'CheckBox', 'ToggleButton',
+           'RadioButton', 'Image', 'ProgressBar', 'StatusBar', 'TreeView',
+           'WebView', 'ContainerBase', 'Box', 'HBox', 'VBox', 'Frame',
+           'Expander', 'TabWidget', 'StackWidget', 'MDIWidget', 'ScrollArea',
+           'Splitter', 'GridBox', 'Toolbar', 'MenuAction',
+           'Menu', 'Menubar', 'TopLevelMixin', 'TopLevel', 'Application',
+           'Dialog', 'SaveDialog', 'DragPackage', 'WidgetMoveEvent',
+           'name_mangle', 'make_widget', 'hadjust', 'build_info', 'wrap',
+           'has_webkit']
+
+
 class WidgetError(Exception):
     """For errors thrown in this module."""
     pass
 
-
 # (see TabWidget)
 _widget_move_event = None
 _app = None
 
+
 # BASE
 
 class WidgetBase(Callback.Callbacks):
@@ -47,18 +72,27 @@ class WidgetBase(Callback.Callbacks):
         self.widget.set_sensitive(tf)
 
     def get_size(self):
-        window = self.widget.get_window()
-        if window is not None:
+        try:
             rect = self.widget.get_allocation()
-            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
-            return wd, ht
+            # x, y = rect.x, rect.y
+            wd, ht = rect.width, rect.height
 
-        # window isn't realized yet--return request size
-        req = self.widget.get_size_request()
-        wd, ht = req
-        wd, ht = max(0, wd), max(0, ht)
+        except Exception as e:
+            # window maybe isn't realized yet--try other ways
+            min_req, nat_req = self.widget.get_preferred_size()
+            wd, ht = nat_req.width, nat_req.height
+            # req = self.widget.get_size_request()
+            # wd, ht = req
+
+            # wd, ht = max(1, wd), max(1, ht)
         return wd, ht
 
+    def get_pos(self):
+        rect = self.widget.get_allocation()
+        x, y = rect.x, rect.y
+        # width, height = rect.width, rect.height
+        return x, y
+
     def get_app(self):
         return _app
 
@@ -66,7 +100,7 @@ class WidgetBase(Callback.Callbacks):
         self.widget.destroy()
 
     def show(self):
-        #self.widget.show()
+        # self.widget.show()
         self.widget.show_all()
 
     def hide(self):
@@ -77,6 +111,8 @@ class WidgetBase(Callback.Callbacks):
 
     def resize(self, width, height):
         self.widget.set_size_request(width, height)
+        # hackish way to allow the widget to be resized down again later
+        # GObject.idle_add(self.widget.set_size_request, -1, -1)
 
     def get_font(self, font_family, point_size):
         font = GtkHelp.get_font(font_family, point_size)
@@ -86,6 +122,7 @@ class WidgetBase(Callback.Callbacks):
         # this is for compatibility with Qt widgets
         pass
 
+
 # BASIC WIDGETS
 
 class TextEntry(WidgetBase):
@@ -144,6 +181,7 @@ class TextEntry(WidgetBase):
         self.widget.set_width_chars(numchars)
         pass
 
+
 class TextEntrySet(WidgetBase):
     def __init__(self, text='', editable=True):
         super(TextEntrySet, self).__init__()
@@ -180,13 +218,14 @@ class TextEntrySet(WidgetBase):
         self.widget.modify_font(font)
 
     def set_length(self, numchars):
-        #self.widget.set_width_chars(numchars)
+        # self.widget.set_width_chars(numchars)
         pass
 
     def set_enabled(self, tf):
         super(TextEntrySet, self).set_enabled(tf)
         self.entry.set_sensitive(tf)
 
+
 class TextArea(WidgetBase):
     def __init__(self, wrap=False, editable=False):
         super(TextArea, self).__init__()
@@ -220,12 +259,12 @@ class TextArea(WidgetBase):
 
         end = buf.get_end_iter()
         mark = buf.get_insert()
-        #self.tw.scroll_to_iter(end, 0.5)
+        # self.tw.scroll_to_iter(end, 0.5)
         # NOTE: this was causing a segfault if the text widget is
         # not mapped yet!  Seems to be fixed in recent versions of
         # gtk
         buf.move_mark(mark, end)
-        res = self.tw.scroll_to_mark(mark, 0.2, False, 0.0, 0.0)
+        res = self.tw.scroll_to_mark(mark, 0.2, False, 0.0, 0.0)  # noqa
 
     def get_text(self):
         buf = self.tw.get_buffer()
@@ -238,7 +277,7 @@ class TextArea(WidgetBase):
         if numlines > self.histlimit:
             rmcount = int(numlines - self.histlimit)
             start = buf.get_iter_at_line(0)
-            end   = buf.get_iter_at_line(rmcount)
+            end = buf.get_iter_at_line(rmcount)
             buf.delete(start, end)
 
     def clear(self):
@@ -267,6 +306,7 @@ class TextArea(WidgetBase):
         else:
             self.tw.set_wrap_mode(Gtk.WrapMode.NONE)
 
+
 class Label(WidgetBase):
     def __init__(self, text='', halign='left', style='normal', menu=None):
         super(Label, self).__init__()
@@ -327,6 +367,7 @@ class Label(WidgetBase):
         if fg is not None:
             self.label.modify_fg(Gtk.StateType.NORMAL, Gdk.color_parse(fg))
 
+
 class Button(WidgetBase):
     def __init__(self, text=''):
         super(Button, self).__init__()
@@ -475,10 +516,10 @@ class Slider(WidgetBase):
 
     def set_tracking(self, tf):
         if tf:
-            #self.widget.set_update_policy(Gtk.UPDATE_CONTINUOUS)
+            # self.widget.set_update_policy(Gtk.UPDATE_CONTINUOUS)
             pass
         else:
-            #self.widget.set_update_policy(Gtk.UPDATE_DISCONTINUOUS)
+            # self.widget.set_update_policy(Gtk.UPDATE_DISCONTINUOUS)
             pass
 
     def set_limits(self, minval, maxval, incr_value=1):
@@ -607,14 +648,15 @@ class Image(WidgetBase):
     def _set_image(self, native_image):
         self.image.set_from_pixbuf(native_image.get_pixbuf())
 
+
 class ProgressBar(WidgetBase):
     def __init__(self):
         super(ProgressBar, self).__init__()
 
         w = Gtk.ProgressBar()
         # GTK3
-        #w.set_orientation(Gtk.Orientation.HORIZONTAL)
-        #w.set_inverted(False)
+        # w.set_orientation(Gtk.Orientation.HORIZONTAL)
+        # w.set_inverted(False)
         self.widget = w
 
     def set_value(self, pct):
@@ -622,6 +664,7 @@ class ProgressBar(WidgetBase):
         self.widget.set_fraction(pct)
         self.widget.set_text("%.2f %%" % (pct * 100.0))
 
+
 class StatusBar(WidgetBase):
     def __init__(self):
         super(StatusBar, self).__init__()
@@ -676,7 +719,7 @@ class TreeView(WidgetBase):
         if self.dragable:
             tv = GtkHelp.MultiDragDropTreeView()
             # enable drag from this widget
-            toImage = [ ( "text/plain", 0, 0 ) ]
+            toImage = [("text/plain", 0, 0)]
             tv.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK,
                                         toImage, Gdk.DragAction.COPY)
             tv.connect("drag-data-get", self._start_drag)
@@ -708,8 +751,8 @@ class TreeView(WidgetBase):
         # create the column headers
         if not isinstance(columns[0], str):
             # columns specifies a mapping
-            headers = [ col[0] for col in columns ]
-            datakeys = [ col[1] for col in columns ]
+            headers = [col[0] for col in columns]
+            datakeys = [col[1] for col in columns]
         else:
             headers = datakeys = columns
 
@@ -740,7 +783,7 @@ class TreeView(WidgetBase):
                 tvc.set_clickable(True)
             if n == 0:
                 fn_data = self._mkcolfn0(kwd)
-                ## cell.set_property('xalign', 1.0)
+                # cell.set_property('xalign', 1.0)
             else:
                 fn_data = self._mkcolfnN(kwd)
             tvc.set_cell_data_func(cell, fn_data)
@@ -886,7 +929,7 @@ class TreeView(WidgetBase):
         treeselection.select_iter(item)
 
     def highlight_path(self, path, onoff, font_color='green'):
-        item = self._path_to_item(path)
+        item = self._path_to_item(path)  # noqa
         # TODO
 
     def _path_to_item(self, path):
@@ -933,13 +976,13 @@ class TreeView(WidgetBase):
             bnch2 = model.get_value(iter2, 0)
             if isinstance(bnch1, str):
                 if isinstance(bnch2, str):
-                    return cmp(bnch1.lower(), bnch2.lower())
+                    return bnch1.lower() == bnch2.lower()  # was using cmp
                 return 0
             val1, val2 = bnch1[idx], bnch2[idx]
             if isinstance(val1, str):
                 val1 = val1.lower()
                 val2 = val2.lower()
-            res = cmp(val1, val2)
+            res = val1 == val2  # was using nonexistent cmp
             return res
         return fn
 
@@ -972,12 +1015,40 @@ class TreeView(WidgetBase):
         return fn
 
     def _start_drag(self, treeview, context, selection,
-                         info, timestamp):
+                    info, timestamp):
         res_dict = self.get_selected()
         drag_pkg = DragPackage(self.tv, selection)
         self.make_callback('drag-start', drag_pkg, res_dict)
         drag_pkg.start_drag()
 
+
+class WebView(WidgetBase):
+    def __init__(self):
+        if not has_webkit:
+            raise NotImplementedError("Missing webkit")
+
+        super(WebView, self).__init__()
+        self.widget = WebKit.WebView()
+
+    def load_url(self, url):
+        self.widget.open(url)
+
+    def load_html_string(self, html_string):
+        self.widget.load_string(html_string, 'text/html', 'utf-8', 'file://')
+
+    def go_back(self):
+        self.widget.go_back()
+
+    def go_forward(self):
+        self.widget.go_forward()
+
+    def reload_page(self):
+        self.widget.reload()
+
+    def stop_loading(self):
+        self.widget.stop_loading()
+
+
 # CONTAINERS
 
 class ContainerBase(WidgetBase):
@@ -995,7 +1066,7 @@ class ContainerBase(WidgetBase):
             childw.destroy()
 
     def remove(self, w, delete=False):
-        if not w in self.children:
+        if w not in self.children:
             raise KeyError("Widget is not a child of this container")
         self.children.remove(w)
 
@@ -1050,10 +1121,12 @@ class Box(ContainerBase):
         self.widget.pack_start(child_w, expand, True, 0)
         self.widget.show_all()
 
+
 class VBox(Box):
     def __init__(self):
         super(VBox, self).__init__(orientation='vertical')
 
+
 class HBox(Box):
     def __init__(self):
         super(HBox, self).__init__(orientation='horizontal')
@@ -1098,17 +1171,17 @@ class TabWidget(ContainerBase):
         self.detachable = detachable
 
         nb = GtkHelp.Notebook()
-        #nb = Gtk.Notebook()
+        # nb = Gtk.Notebook()
         nb.set_show_border(False)
         nb.set_scrollable(True)
         # Allows drag-and-drop between notebooks
-        #nb.set_group_id(group)  # in gtk3?
+        # nb.set_group_id(group)  # in gtk3?
         if self.detachable:
             nb.connect("create-window", self._tab_detach_cb)
         nb.connect("page-added", self._tab_insert_cb)
         nb.connect("page-removed", self._tab_remove_cb)
         nb.sconnect("switch-page", self._cb_redirect)
-        #nb.connect("switch-page", self._cb_redirect)
+        # nb.connect("switch-page", self._cb_redirect)
         self.widget = nb
         self.set_tab_position(tabpos)
 
@@ -1132,7 +1205,7 @@ class TabWidget(ContainerBase):
         # (native widget already has been removed by gtk)
         self.children.remove(child)
 
-        #nchild_w.unparent()
+        # nchild_w.unparent()
         self.make_callback('page-detach', child)
 
     def _tab_insert_cb(self, nbw, nchild_w, page_num):
@@ -1202,52 +1275,39 @@ class TabWidget(ContainerBase):
         else:
             evbox.modify_bg(Gtk.StateType.NORMAL, None)
 
+
 class StackWidget(TabWidget):
     def __init__(self):
         super(StackWidget, self).__init__()
 
         nb = self.widget
-        #nb.set_scrollable(False)
+        # nb.set_scrollable(False)
         nb.set_show_tabs(False)
         nb.set_show_border(False)
 
-class MDIWidget(TabWidget):
+
+class MDIWidget(ContainerBase):
 
     def __init__(self, tabpos='top', mode='tabs'):
-        super(MDIWidget, self).__init__(tabpos=tabpos)
+        super(MDIWidget, self).__init__()
 
-        self.mode = 'tabs'
-        self.true_mdi = False
-
-    def get_mode(self):
-        return self.mode
-
-    def set_mode(self, mode):
-        pass
-
-    def tile_panes(self):
-        pass
-
-    def cascade_panes(self):
-        pass
-
-    def use_tabs(self, tf):
-        pass
-
-
-class MDIWidget2(ContainerBase):
-
-    def __init__(self, tabpos='top', mode='tabs'):
-        super(MDIWidget2, self).__init__()
-
-        self.mode = 'tabs'
-        self.true_mdi = False
+        self.mode = 'mdi'
+        self.true_mdi = True
 
+        # TODO: currently scrollbars are not working
         sw = Gtk.ScrolledWindow()
         sw.set_border_width(2)
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         self.widget = sw
-        self.mdi_w = GtkHelp.MDIWidget()
+        w = GtkHelp.MDIWidget()
+        self.mdi_w = w
+        w._move_page = w.move_page
+        w.move_page = self._window_moved
+        w._resize_page = w.resize_page
+        w.resize_page = self._window_resized
+
+        sw.set_hadjustment(self.mdi_w.get_hadjustment())
+        sw.set_vadjustment(self.mdi_w.get_vadjustment())
         sw.add(self.mdi_w)
 
     def get_mode(self):
@@ -1260,15 +1320,50 @@ class MDIWidget2(ContainerBase):
         self.add_ref(child)
         child_w = child.get_widget()
         label = Gtk.Label(title)
-        self.mdi_w.append_page(child_w, label)
-        #self.mdi_w.show_all()
+        subwin = self.mdi_w.append_page(child_w, label)
+        # self.mdi_w.show_all()
         # attach title to child
         child.extdata.tab_title = title
 
+        # does child have a previously saved size
+        size = child.extdata.get('mdi_size', None)
+        if size is not None:
+            wd, ht = size
+            self.mdi_w.resize_page(subwin, wd, ht)
+
+        # does child have a previously saved position
+        pos = child.extdata.get('mdi_pos', None)
+        if pos is not None:
+            x, y = pos
+            self.mdi_w.move_page(subwin, x, y)
+
+    def _remove(self, childw, delete=False):
+        self.mdi_w.remove(childw)
+        if delete:
+            childw.destroy()
+
     def _cb_redirect(self, nbw, gptr, index):
         child = self.index_to_widget(index)
         self.make_callback('page-switch', child)
 
+    def _window_resized(self, subwin, wd, ht):
+        self.mdi_w._resize_page(subwin, wd, ht)
+
+        # save size
+        nchild = subwin.widget
+        child = self._native_to_child(nchild)
+        child.extdata.mdi_size = (wd, ht)
+        return True
+
+    def _window_moved(self, subwin, x, y):
+        self.mdi_w._move_page(subwin, x, y)
+
+        # save position
+        nchild = subwin.widget
+        child = self._native_to_child(nchild)
+        child.extdata.mdi_pos = (x, y)
+        return True
+
     def get_index(self):
         return self.mdi_w.get_current_page()
 
@@ -1292,6 +1387,7 @@ class MDIWidget2(ContainerBase):
     def use_tabs(self, tf):
         pass
 
+
 class ScrollArea(ContainerBase):
     def __init__(self):
         super(ScrollArea, self).__init__()
@@ -1306,7 +1402,8 @@ class ScrollArea(ContainerBase):
 
     def _resize_cb(self, widget, allocation):
         rect = widget.get_allocation()
-        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        # x, y = rect.x, rect.y
+        width, height = rect.width, rect.height
         self.make_callback('configure', width, height)
         return True
 
@@ -1340,6 +1437,7 @@ class Splitter(ContainerBase):
             w = Gtk.HPaned()
         else:
             w = Gtk.VPaned()
+        w.set_wide_handle(True)
         return w
 
     def add_widget(self, child):
@@ -1529,19 +1627,33 @@ class Menu(ContainerBase):
         super(Menu, self).__init__()
 
         self.widget = Gtk.Menu()
+        self.menus = Bunch.Bunch(caseless=True)
         self.widget.show()
 
     def add_widget(self, child):
         menuitem_w = child.get_widget()
         self.widget.append(menuitem_w)
         self.add_ref(child)
-        #self.widget.show_all()
+        # self.widget.show_all()
 
     def add_name(self, name, checkable=False):
         child = MenuAction(text=name, checkable=checkable)
         self.add_widget(child)
         return child
 
+    def add_menu(self, name):
+        item_w = Gtk.MenuItem(label=name)
+        child = Menu()
+        self.add_ref(child)
+        self.menus[name] = child
+        item_w.set_submenu(child.get_widget())
+        self.widget.append(item_w)
+        item_w.show()
+        return child
+
+    def get_menu(self, name):
+        return self.menus[name]
+
     def add_separator(self):
         sep = Gtk.SeparatorMenuItem()
         self.widget.append(sep)
@@ -1551,7 +1663,7 @@ class Menu(ContainerBase):
         menu = self.widget
         menu.show_all()
         if six.PY2:
-            now = long(0)
+            now = long(0)  # noqa
         else:
             now = int(0)
         menu.popup(None, None, None, None, 0, now)
@@ -1593,8 +1705,9 @@ class TopLevelMixin(object):
         self.widget.connect("destroy", self._quit)
         self.widget.connect("delete_event", self._close_event)
         self.widget.connect("window_state_event", self._window_event)
+        self.widget.connect("configure-event", self._configure_event)
 
-        if not title is None:
+        if title is not None:
             self.widget.set_title(title)
 
         self.enable_callback('close')
@@ -1609,67 +1722,108 @@ class TopLevelMixin(object):
         self.close()
 
     def _close_event(self, widget, event):
-        self.close()
-        # don't automatically destroy window
-        return True
+        try:
+            self.close()
+
+        finally:
+            # don't automatically destroy window
+            return True
 
     def _window_event(self, widget, event):
         if ((event.changed_mask & Gdk.WindowState.FULLSCREEN) or
-            (event.changed_mask & Gdk.WindowState.MAXIMIZED)):
+                (event.changed_mask & Gdk.WindowState.MAXIMIZED)):
             self._fullscreen = True
         else:
             self._fullscreen = False
 
+    def _configure_event(self, widget, event):
+        x, y, width, height = event.x, event.y, event.width, event.height
+        x, y = self.widget.translate_coordinates(self.widget, x, y)
+        self.extdata.setvals(x=x, y=y, width=width, height=height)
+        return False
+
     def close(self):
-        ## try:
-        ##     self.widget.destroy()
-        ## except Exception as e:
-        ##     pass
-        #self.widget = None
+        # try:
+        #     self.widget.destroy()
+        # except Exception as e:
+        #     pass
+        # self.widget = None
 
         self.make_callback('close')
 
+    def get_size(self):
+        try:
+            rect = self.widget.get_allocation()
+            # x, y = rect.x, rect.y
+            wd, ht = rect.width, rect.height
+
+        except Exception as e:
+            # window maybe isn't realized yet--try other ways
+            # req = self.widget.get_size_request()
+            # wd, ht = req
+            min_req, nat_req = self.widget.get_preferred_size()
+            wd, ht = nat_req.width, nat_req.height
+            ed = self.extdata
+            wd, ht = ed.get('width', wd), ed.get('height', ht)
+
+        return wd, ht
+
+    def get_pos(self):
+        res = None
+        window = self.widget.get_window()
+        if window is not None:
+            res = window.get_origin()
+            if isinstance(res, tuple) and len(res) == 2:
+                return res
+
+        ed = self.extdata
+        x, y = ed.get('x', None), ed.get('y', None)
+        return x, y
+
     def raise_(self):
         window = self.widget.get_window()
-        ## if window:
-        ##     if hasattr(window, 'present'):
-        ##         # gtk3 ?
-        ##         window.present()
-        ##     else:
-        ##         # gtk2
-        ##         window.show()
         window.raise_()
 
     def lower(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.lower()
 
-    def resize(self, width, height):
-        # TODO: use window.set_default_size() ?
-        self.widget.set_size_request(width, height)
-
     def focus(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.focus()
 
     def move(self, x, y):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.move(x, y)
 
     def maximize(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.maximize()
 
     def unmaximize(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.unmaximize()
 
     def fullscreen(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.fullscreen()
 
     def unfullscreen(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.unfullscreen()
 
     def is_fullscreen(self):
@@ -1677,10 +1831,14 @@ class TopLevelMixin(object):
 
     def iconify(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.iconify()
 
     def uniconify(self):
         window = self.widget.get_window()
+        if window is None:
+            return
         window.deiconify()
 
     def set_title(self, title):
@@ -1725,14 +1883,14 @@ class Application(Callback.Callbacks):
         except:
             self.screen_wd = 1600
             self.screen_ht = 1200
-        ## self.logger.debug("screen dimensions %dx%d" % (
-        ##     self.screen_wd, self.screen_ht))
+        # self.logger.debug("screen dimensions %dx%d" % (
+        #     self.screen_wd, self.screen_ht))
 
         _app = self
 
         # supposedly needed for GObject < 3.10.2
         GObject.threads_init()
-        ## self._time_save = time.time()
+        # self._time_save = time.time()
 
         for name in ('shutdown', ):
             self.enable_callback(name)
@@ -1752,16 +1910,15 @@ class Application(Callback.Callbacks):
                 # lost--we want to know whether the process_event loop
                 # is running, so ping periodically if events are showing
                 # up
-                ## cur_time = time.time()
-                ## if cur_time - self._time_save > 10.0:
-                ##     self.logger.info("process_events ping!")
-                ##     self._time_save = cur_time
+                # cur_time = time.time()
+                # if cur_time - self._time_save > 10.0:
+                #     self.logger.info("process_events ping!")
+                #     self._time_save = cur_time
 
             except Exception as e:
                 self.logger.error("Exception in main_iteration() loop: %s" %
                                   (str(e)))
 
-
     def process_end(self):
         pass
 
@@ -1862,7 +2019,8 @@ class SaveDialog(object):
 
         if response == Gtk.ResponseType.OK:
             path = self.widget.get_filename()
-            if self.selectedfilter is not None and not path.endswith(self.selectedfilter):
+            if (self.selectedfilter is not None and
+                    not path.endswith(self.selectedfilter)):
                 path += self.selectedfilter
             self.widget.destroy()
             return path
@@ -1870,6 +2028,7 @@ class SaveDialog(object):
             self.widget.destroy()
             return None
 
+
 class DragPackage(object):
     def __init__(self, src_widget, selection):
         self.src_widget = src_widget
@@ -1882,6 +2041,7 @@ class DragPackage(object):
     def start_drag(self):
         pass
 
+
 class WidgetMoveEvent(object):
     def __init__(self, src_widget, child):
         self.src_widget = src_widget
@@ -1894,6 +2054,7 @@ class WidgetMoveEvent(object):
     def reject(self):
         self._result = False
 
+
 # MODULE FUNCTIONS
 
 def name_mangle(name, pfx=''):
@@ -1905,6 +2066,7 @@ def name_mangle(name, pfx=''):
             newname.append(c)
     return pfx + ''.join(newname)
 
+
 def make_widget(title, wtype):
     if wtype == 'label':
         w = Label(title)
@@ -1914,7 +2076,7 @@ def make_widget(title, wtype):
         w.label.set_alignment(0.05, 0.95)
     elif wtype == 'entry':
         w = TextEntry()
-        #w.get_widget().set_width_chars(12)
+        # w.get_widget().set_width_chars(12)
     elif wtype == 'entryset':
         w = TextEntrySet()
     elif wtype == 'combobox':
@@ -2011,36 +2173,4 @@ def wrap(native_widget):
     return wrapper
 
 
-def get_orientation(container):
-    if not hasattr(container, 'size'):
-        return 'vertical'
-    (wd, ht) = container.size
-    ## wd, ht = container.get_size()
-    #print('container size is %dx%d' % (wd, ht))
-    if wd < ht:
-        return 'vertical'
-    else:
-        return 'horizontal'
-
-def get_oriented_box(container, scrolled=True, fill=False):
-    orientation = get_orientation(container)
-
-    if orientation == 'vertical':
-        box1 = VBox()
-        box2 = VBox()
-    else:
-        box1 = HBox()
-        box2 = VBox()
-
-    box2.add_widget(box1, stretch=1)
-    if not fill:
-        box2.add_widget(Label(''), stretch=1)
-    if scrolled:
-        sw = ScrollArea()
-        sw.set_widget(box2)
-    else:
-        sw = box2
-
-    return box1, sw, orientation
-
-#END
+# END
