@@ -1,3 +1,4 @@
+
 #
 # GtkHelp.py -- customized Gtk2 widgets
 #
@@ -5,9 +6,11 @@
 # Please see the file LICENSE.txt for details.
 #
 from __future__ import print_function
-import sys, os.path
+import sys
+import os.path
+import math
 
-from ginga.misc import Bunch
+from ginga.misc import Bunch, Callback
 import ginga.toolkit
 
 import gtk
@@ -213,7 +216,8 @@ class Notebook(WidgetMask, gtk.Notebook):
         gtk.Notebook.__init__(self, *args, **kwdargs)
 
     def set_group_id(self, id):
-        super(Notebook, self).set_group_name(str(id))
+        if hasattr(self, 'set_group_name'):
+            super(Notebook, self).set_group_name(str(id))
 
     def set_current_page(self, new_idx):
         old_idx = self.get_current_page()
@@ -275,6 +279,7 @@ class MDIWidget(gtk.Layout):
         self.selected_child = None
         self.kbdmouse_mask = 0
         self.cascade_offset = 50
+        self.minimized_width = 150
 
         self.connect("motion_notify_event", self.motion_notify_event)
         self.connect("button_press_event", self.button_press_event)
@@ -290,24 +295,42 @@ class MDIWidget(gtk.Layout):
                         | gtk.gdk.KEY_PRESS_MASK
                         | gtk.gdk.KEY_RELEASE_MASK
                         | gtk.gdk.POINTER_MOTION_MASK
-                        #| gtk.gdk.POINTER_MOTION_HINT_MASK
+                        | gtk.gdk.POINTER_MOTION_HINT_MASK
                         | gtk.gdk.SCROLL_MASK)
 
+        self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray50"))
+
     def append_page(self, widget, label):
         vbox = gtk.VBox()
         vbox.set_border_width(4)
+        hbox = gtk.HBox()
+        close = gtk.Button("x")
+        maxim = gtk.Button("^")
+        minim = gtk.Button("v")
+        hbox.pack_start(close, False, False, 0)
+        hbox.pack_start(minim, False, False, 0)
+        hbox.pack_start(maxim, False, False, 0)
+
         evbox = gtk.EventBox()
         evbox.add(label)
-        #evbox.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("yellow"))
-        evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("skyblue"))
-        vbox.pack_start(evbox, fill=False, expand=False, padding=0)
+        evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray90"))
+        hbox.pack_start(evbox, True, True, 2)
+
+        vbox.pack_start(hbox, fill=False, expand=False, padding=0)
         vbox.pack_start(widget, fill=True, expand=True, padding=4)
 
+        # what size does the widget want to be?
+        x, y, wd, ht = widget.get_allocation()
+        ## wd = widget.get_preferred_width()
+        ## ht = widget.get_preferred_height()
+        ## wd, ht = widget.get_size_request()
+        wd, ht = max(wd, 300), max(ht, 300)
+
         frame = gtk.EventBox()
-        frame.set_size_request(300, 300)
+        frame.set_size_request(wd, ht)
         frame.props.visible_window = True
         frame.set_border_width(0)
-        frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("palegreen1"))
+        frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray70"))
 
         frame.add(vbox)
         frame.show_all()
@@ -324,7 +347,12 @@ class MDIWidget(gtk.Layout):
 
         evbox.connect("button_press_event", self.select_child_cb, subwin)
         frame.connect("button_press_event", self.start_resize_cb, subwin)
+        maxim.connect('clicked', lambda *args: self.maximize_page(subwin))
+        minim.connect('clicked', lambda *args: self.minimize_page(subwin))
+        close.connect('clicked', lambda *args: self.close_page(subwin))
+
         self.put(frame, self.cascade_offset, self.cascade_offset)
+        return subwin
 
     def set_tab_reorderable(self, w, tf):
         pass
@@ -383,6 +411,16 @@ class MDIWidget(gtk.Layout):
         x, y, width, height = widget.get_allocation()
         return width, height
 
+    def update_subwin_position(self, subwin):
+        rect = subwin.frame.get_allocation()
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+        subwin.x, subwin.y = x, y
+
+    def update_subwin_size(self, subwin):
+        rect = subwin.frame.get_allocation()
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+        subwin.width, subwin.height = wd, ht
+
     def raise_widget(self, subwin):
         frame = subwin.frame
         # Hack to bring widget to the top--no documentation on any other
@@ -403,8 +441,8 @@ class MDIWidget(gtk.Layout):
             self.set_current_page(idx)
 
         self.selected_child = Bunch.Bunch(subwin=subwin, action='move',
-                                          cr = self.setup_cr(self.bin_window),
                                           x_origin=x, y_origin=y, dx=ex, dy=ey)
+
         return True
 
     def start_resize_cb(self, widget, event, subwin):
@@ -412,26 +450,17 @@ class MDIWidget(gtk.Layout):
         x, y = self.get_widget_position(subwin.frame)
         subwin.x, subwin.y = x, y
         self.selected_child = Bunch.Bunch(subwin=subwin, action='resize',
-                                          cr = self.setup_cr(self.bin_window),
                                           x_origin=x, y_origin=y, dx=ex, dy=ey)
         return True
 
     def button_press_event(self, widget, event):
-        # event.button, event.x, event.y
         x = event.x; y = event.y
         button = self.kbdmouse_mask
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
         return True
 
-    def setup_cr(self, drawable):
-        cr = drawable.cairo_create()
-        cr.set_line_width(2)
-        cr.set_dash([ 3.0, 4.0, 6.0, 4.0], 5.0)
-        return cr
-
     def button_release_event(self, widget, event):
-        # event.button, event.x, event.y
         x = event.x_root; y = event.y_root
         button = self.kbdmouse_mask
         if event.button != 0:
@@ -442,22 +471,19 @@ class MDIWidget(gtk.Layout):
             if bnch.action == 'move':
                 x = int(subwin.x + (x - bnch.dx))
                 y = int(subwin.y + (y - bnch.dy))
-                self.move(subwin.frame, x, y)
-                subwin.x, subwin.y = self.get_widget_position(subwin.frame)
+                self.move_page(subwin, x, y)
+
             elif bnch.action == 'resize':
                 wd = int(subwin.width + (x - bnch.dx))
                 ht = int(subwin.height + (y - bnch.dy))
-                subwin.frame.set_size_request(wd, ht)
-                subwin.width, subwin.height = self.get_widget_size(subwin.frame)
+                self.resize_page(subwin, wd, ht)
+
             self.selected_child = None
         return True
 
     def motion_notify_event(self, widget, event):
         button = self.kbdmouse_mask
-        if event.is_hint:
-            return
-        else:
-            x, y, state = event.x_root, event.y_root, event.state
+        x, y, state = event.x_root, event.y_root, event.state
 
         if state & gtk.gdk.BUTTON1_MASK:
             button |= 0x1
@@ -472,26 +498,86 @@ class MDIWidget(gtk.Layout):
             if bnch.action == 'move':
                 x = int(subwin.x + (x - bnch.dx))
                 y = int(subwin.y + (y - bnch.dy))
+                # this works better if it is not self.move_page()
                 self.move(subwin.frame, x, y)
+
             elif bnch.action == 'resize':
                 wd = int(subwin.width + (x - bnch.dx))
                 ht = int(subwin.height + (y - bnch.dy))
+                # this works better if it is not self.resize_page()
                 subwin.frame.set_size_request(wd, ht)
+
         return True
 
     def tile_pages(self):
-        pass
+        # calculate number of rows and cols, try to maintain a square
+        # TODO: take into account the window geometry
+        num_widgets = len(self.children)
+        rows = int(round(math.sqrt(num_widgets)))
+        cols = rows
+        if rows**2 < num_widgets:
+            cols += 1
+
+        # find out how big each window should be
+        x, y, width, height = self.get_allocation()
+        wd, ht = width // cols, height // rows
+
+        # and move and resize them into place
+        for i in range(0, rows):
+            for j in range(0, cols):
+                index = i*cols + j
+                if index < num_widgets:
+                    subwin = self.children[index]
+
+                    self.resize_page(subwin, wd, ht)
+
+                    x, y = j * wd, i * ht
+                    self.move_page(subwin, x, y)
+
+                    self.raise_widget(subwin)
 
     def cascade_pages(self):
         x, y = 0, 0
         for subwin in self.children:
-            self.move(subwin.frame, x, y)
+            self.move_page(subwin, x, y)
+            self.raise_widget(subwin)
             x += self.cascade_offset
             y += self.cascade_offset
 
     def use_tabs(self, tf):
         pass
 
+    def move_page(self, subwin, x, y):
+        self.move(subwin.frame, x, y)
+        subwin.x, subwin.y = x, y
+
+    def resize_page(self, subwin, wd, ht):
+        subwin.frame.set_size_request(wd, ht)
+        subwin.width, subwin.height = wd, ht
+
+    def maximize_page(self, subwin):
+        x, y, wd, ht = self.get_allocation()
+
+        self.raise_widget(subwin)
+        self.resize_page(subwin, wd, ht)
+        self.move_page(subwin, 0, 0)
+
+    def minimize_page(self, subwin):
+        rect = self.get_allocation()
+        _x, _y, width, height = rect.x, rect.y, rect.width, rect.height
+
+        rect = subwin.frame.get_allocation()
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+
+        rect = subwin.label.get_allocation()
+        _x, _y, wd, ht = rect.x, rect.y, rect.width, rect.height
+
+        self.resize_page(subwin, self.minimized_width, ht)
+        self.move_page(subwin, x, height-ht)
+        #self.lower_widget(subwin)
+
+    def close_page(self, subwin):
+        pass
 
 class FileSelection(object):
 
@@ -552,36 +638,45 @@ class DirectorySelection(FileSelection):
 class Timer(object):
     """Abstraction of a GUI-toolkit implemented timer."""
 
-    def __init__(self, ival_sec, expire_cb, data=None):
-        """Create a timer set to expire after `ival_sec` and which will
-        call the callable `expire_cb` when it expires.
+class Timer(Callback.Callbacks):
+    """Abstraction of a GUI-toolkit implemented timer."""
+
+    def __init__(self, duration=0.0):
+        """Create a timer set to expire after `duration` sec.
         """
-        self.ival_sec = ival_sec
-        self.cb = expire_cb
-        self.data = data
+        super(Timer, self).__init__()
+
+        self.duration = duration
+        # For storing aritrary data with timers
+        self.data = Bunch.Bunch()
+
         self._timer = None
 
-    def start(self, ival_sec=None):
-        """Start the timer.  If `ival_sec` is not None, it should
+        for name in ('expired', 'canceled'):
+            self.enable_callback(name)
+
+    def start(self, duration=None):
+        """Start the timer.  If `duration` is not None, it should
         specify the time to expiration in seconds.
         """
-        if ival_sec is None:
-            ival_sec = self.ival_sec
+        if duration is None:
+            duration = self.duration
 
-        self.cancel()
+        self.set(duration)
+
+    def set(self, duration):
+
+        self.stop()
 
         # Gtk timer set in milliseconds
-        time_ms = int(ival_sec * 1000.0)
+        time_ms = int(duration * 1000.0)
         self._timer = gobject.timeout_add(time_ms, self._redirect_cb)
 
     def _redirect_cb(self):
         self._timer = None
-        self.cb(self)
+        self.make_callback('expired')
 
-    def cancel(self):
-        """Cancel this timer.  If the timer is not running, there
-        is no error.
-        """
+    def stop(self):
         try:
             if self._timer is not None:
                 gobject.source_remove(self._timer)
@@ -589,6 +684,15 @@ class Timer(object):
         except:
             pass
 
+    def cancel(self):
+        """Cancel this timer.  If the timer is not running, there
+        is no error.
+        """
+        self.stop()
+        self.make_callback('canceled')
+
+    clear = cancel
+
 
 def combo_box_new_text():
     liststore = gtk.ListStore(gobject.TYPE_STRING)
