@@ -8,7 +8,8 @@ import time
 import math
 
 from ginga.misc import Bunch, Callback
-from ginga.gw import Widgets, GwHelp, Viewers
+from ginga.gw import Widgets, Viewers
+
 
 class Desktop(Callback.Callbacks):
 
@@ -23,16 +24,18 @@ class Desktop(Callback.Callbacks):
         self.workspace = Bunch.caselessDict()
 
         self.toplevels = []
+        self.node = {}
+        self.node_idx = 0
         self._cur_dialogs = []
 
-        for name in ('page-switch', 'all-closed'):
+        for name in ('page-switch', 'page-close', 'all-closed'):
             self.enable_callback(name)
 
     # --- Workspace Handling ---
 
     def make_ws(self, name, group=1, show_tabs=True, show_border=False,
-                detachable=False, tabpos=None, scrollable=True, closeable=False,
-                wstype='tabs', use_toolbar=False):
+                detachable=False, tabpos=None, scrollable=True,
+                closeable=False, wstype='tabs', use_toolbar=False):
 
         if (wstype in ('nb', 'ws', 'tabs')) and (not show_tabs):
             wstype = 'stack'
@@ -42,6 +45,7 @@ class Desktop(Callback.Callbacks):
                        use_toolbar=use_toolbar)
         ws.add_callback('page-switch', self.switch_page_cb)
         ws.add_callback('page-detach', self.page_detach_cb)
+        ws.add_callback('page-close', self.page_close_cb)
 
         nb = ws.nb
 
@@ -115,10 +119,11 @@ class Desktop(Callback.Callbacks):
                 tabname = 'tab%d' % self.tabcount
 
         ws.add_tab(widget, title=labelname)
-        self.tab[tabname] = Bunch.Bunch(widget=widget, name=labelname,
-                                        tabname=tabname, data=data,
-                                        group=group, wsname=wsname)
-        return tabname
+        bnch = Bunch.Bunch(widget=widget, name=labelname,
+                           tabname=tabname, data=data,
+                           group=group, wsname=wsname)
+        self.tab[tabname] = bnch
+        return bnch
 
     def _find_nb(self, tabname):
         widget = self.tab[tabname].widget
@@ -141,7 +146,7 @@ class Desktop(Callback.Callbacks):
         # should be raised
         l = []
         name = tabname.lower()
-        while self.tab.has_key(name):
+        while name in self.tab:
             bnch = self.tab[name]
             l.insert(0, name)
             name = bnch.wsname.lower()
@@ -169,7 +174,7 @@ class Desktop(Callback.Callbacks):
     def show_dialog(self, dialog):
         dialog.show()
         # save a handle so widgets aren't garbage collected
-        if not dialog in self._cur_dialogs:
+        if dialog not in self._cur_dialogs:
             self._cur_dialogs.append(dialog)
 
     def hide_dialog(self, dialog):
@@ -208,13 +213,13 @@ class Desktop(Callback.Callbacks):
         root = self.app.make_window()
 
         # TODO: this needs to be more sophisticated
-        ## root.set_title(wsname)
+        # root.set_title(wsname)
         ws = self.make_ws(wsname, wstype='tabs', use_toolbar=True)
 
         vbox = Widgets.VBox()
         vbox.set_border_width(0)
 
-        vbox.add_widget(bnch.widget)
+        vbox.add_widget(ws.widget)
         root.set_widget(vbox)
 
         root.resize(width, height)
@@ -223,7 +228,8 @@ class Desktop(Callback.Callbacks):
 
         if x is not None:
             root.move(x, y)
-        return bnch
+
+        return ws
 
     def _mk_take_tab_cb(self, tabname, to_nb):
         def _foo():
@@ -240,7 +246,7 @@ class Desktop(Callback.Callbacks):
         if num_children == 0:
             del self.workspace[ws.name]
             root = ws.root
-            bnch.root = None
+            ws.root = None
             root.delete()
         return True
 
@@ -268,18 +274,36 @@ class Desktop(Callback.Callbacks):
             self.make_callback('page-switch', bnch.name, bnch.data)
         return False
 
-    def make_desktop(self, layout, widgetDict=None):
-        if widgetDict is None:
-            widgetDict = {}
+    def page_close_cb(self, ws, child):
+        self.logger.debug("page close: %s" % str(child))
+        bnch = self._find_tab(child)
+        if bnch is not None:
+            self.make_callback('page-close', bnch.name, bnch.data)
+        return False
 
-        def process_common_params(widget, inparams):
-            params = Bunch.Bunch(name=None, height=-1, width=-1,
-                                 xpos=-1, ypos=-1, spacing=None,
-                                 wexp=None, hexp=None)
-            params.update(inparams)
+    def make_desktop(self, layout, widget_dict=None):
+        if widget_dict is None:
+            widget_dict = {}
+        self.node = {}
+        self.node_idx = 0
 
-            if params.name:
-                widgetDict[params.name] = widget
+        def process_common_params(kind, widget, params):
+            subst_params = Bunch.Bunch(name=None, height=-1, width=-1,
+                                       xpos=-1, ypos=-1, spacing=None,
+                                       wexp=None, hexp=None)
+            subst_params.update(params)
+            params.update(subst_params)
+
+            if params.name is not None:
+                name = params.name
+                widget_dict[params.name] = widget
+            else:
+                name = '%s_%d' % (kind, self.node_idx)
+                self.node_idx += 1
+
+            bnch = Bunch.Bunch(kind=kind, widget=widget,
+                               name=name, params=params)
+            self.node[name] = bnch
 
             wexp, hexp = params.wexp, params.hexp
 
@@ -298,7 +322,7 @@ class Desktop(Callback.Callbacks):
                     else:
                         width = params.width
                         if wexp is None:
-                            wexp = 1|4
+                            wexp = 1 | 4
                     if params.height < 0:
                         height = widget.get_size()[1]
                         if hexp is None:
@@ -306,7 +330,7 @@ class Desktop(Callback.Callbacks):
                     else:
                         height = params.height
                         if hexp is None:
-                            hexp = 1|4
+                            hexp = 1 | 4
                     widget.resize(width, height)
 
                 # specify expansion policy of widget
@@ -321,16 +345,19 @@ class Desktop(Callback.Callbacks):
                 if params.xpos >= 0:
                     widget.move(params.xpos, params.ypos)
 
-        def make_widget(kind, paramdict, args, pack):
+            return bnch
+
+        def make_widget(kind, params, args, pack):
             kind = kind.lower()
 
             # Process workspace parameters
-            params = Bunch.Bunch(name=None, title=None, height=-1,
-                                 width=-1, group=1, show_tabs=True,
-                                 show_border=False, scrollable=True,
-                                 detachable=False, wstype='tabs',
-                                 tabpos='top', use_toolbar=False)
-            params.update(paramdict)
+            param_dict = Bunch.Bunch(name=None, title=None, height=-1,
+                                     width=-1, group=1, show_tabs=True,
+                                     show_border=False, scrollable=True,
+                                     detachable=False, wstype='tabs',
+                                     tabpos='top', use_toolbar=False)
+            param_dict.update(params)
+            params.update(param_dict)
 
             if kind == 'widget':
                 widget = args[0]
@@ -346,7 +373,7 @@ class Desktop(Callback.Callbacks):
                                   scrollable=params.scrollable,
                                   use_toolbar=params.use_toolbar)
                 widget = ws.widget
-                #debug(widget)
+                # debug(widget)
 
             # If a title was passed as a parameter, then make a frame to
             # wrap the widget using the title.
@@ -357,8 +384,9 @@ class Desktop(Callback.Callbacks):
             else:
                 pack(widget)
 
-            #process_common_params(widget, params)
+            # process_common_params(kind, widget, params)
 
+            res = []
             if (kind in ('ws', 'mdi', 'grid', 'stack')) and (len(args) > 0):
                 # <-- Workspace specified a sub-layout.  We expect a list
                 # of tabname, layout pairs--iterate over these and add them
@@ -369,67 +397,82 @@ class Desktop(Callback.Callbacks):
                         self.add_tab(params.name, w, group,
                                      tabname, tabname.lower())
 
-                    make(layout, pack)
+                    res.append((tabname, make(layout, pack)))
 
-            process_common_params(widget, params)
+            bnch = process_common_params(kind, widget, params)
+            if kind == 'ws':
+                bnch.ws = ws
 
-            #return widget
+            # return [kind, params] + res
+            return [kind, params, res]
 
         # Horizontal adjustable panel
         def hpanel(params, cols, pack):
+            res = []
             if len(cols) >= 2:
                 widget = Widgets.Splitter(orientation='horizontal')
-                process_common_params(widget, params)
+                process_common_params('hpanel', widget, params)
 
                 sizes = []
                 for col in cols:
-                    make(col, lambda w: widget.add_widget(w))
+                    c = make(col, lambda w: widget.add_widget(w))
+                    res.append(c)
 
                     # collect widths to set width of panes
                     params = col[1]
                     if 'width' in params:
                         sizes.append(params['width'])
 
+                sizes = params.get('sizes', sizes)
                 if len(sizes) == len(cols):
                     widget.set_sizes(sizes)
+                    params.update(dict(sizes=sizes))
 
             elif len(cols) == 1:
                 widget = Widgets.HBox()
                 widget.set_border_width(0)
-                process_common_params(widget, params)
+                process_common_params('hbox', widget, params)
 
-                make(cols[0], lambda w: widget.add_widget(w, stretch=1))
-                #widget.show()
+                c = make(cols[0], lambda w: widget.add_widget(w, stretch=1))
+                res.append(c)
+                # widget.show()
 
             pack(widget)
+            return ['hpanel', params] + res
 
         # Vertical adjustable panel
         def vpanel(params, rows, pack):
+            res = []
             if len(rows) >= 2:
                 widget = Widgets.Splitter(orientation='vertical')
-                process_common_params(widget, params)
+                process_common_params('vpanel', widget, params)
 
                 sizes = []
                 for row in rows:
-                    make(row, lambda w: widget.add_widget(w))
+                    r = make(row, lambda w: widget.add_widget(w))
+                    res.append(r)
 
                     # collect heights to set height of panes
                     params = row[1]
                     if 'height' in params:
                         sizes.append(params['height'])
 
+                sizes = params.get('sizes', sizes)
                 if len(sizes) == len(rows):
                     widget.set_sizes(sizes)
+                    params.update(dict(sizes=sizes))
 
             elif len(rows) == 1:
                 widget = Widgets.VBox()
                 widget.set_border_width(0)
-                process_common_params(widget, params)
+                process_common_params('vbox', widget, params)
 
-                make(rows[0], lambda w: widget.add_widget(w, stretch=1))
-                #widget.show()
+                r = make(rows[0], lambda w: widget.add_widget(w, stretch=1))
+                res.append(r)
+                # widget.show()
 
             pack(widget)
+            return ['vpanel', params] + res
 
         # Horizontal fixed array
         def hbox(params, cols, pack):
@@ -437,6 +480,7 @@ class Desktop(Callback.Callbacks):
             widget.set_border_width(0)
             widget.set_spacing(0)
 
+            res = []
             for dct in cols:
                 if isinstance(dct, dict):
                     stretch = dct.get('stretch', 0)
@@ -446,10 +490,14 @@ class Desktop(Callback.Callbacks):
                     stretch = 0
                     col = dct
                 if col is not None:
-                    make(col, lambda w: widget.add_widget(w,
-                                                          stretch=stretch))
-            process_common_params(widget, params)
+                    c = make(col, lambda w: widget.add_widget(w,
+                                                              stretch=stretch))
+                    c = dict(col=c, stretch=stretch)
+                    res.append(c)
+
+            process_common_params('hbox', widget, params)
             pack(widget)
+            return ['hbox', params] + res
 
         # Vertical fixed array
         def vbox(params, rows, pack):
@@ -457,6 +505,7 @@ class Desktop(Callback.Callbacks):
             widget.set_border_width(0)
             widget.set_spacing(0)
 
+            res = []
             for dct in rows:
                 if isinstance(dct, dict):
                     stretch = dct.get('stretch', 0)
@@ -466,16 +515,20 @@ class Desktop(Callback.Callbacks):
                     stretch = 0
                     row = dct
                 if row is not None:
-                    make(row, lambda w: widget.add_widget(w,
-                                                          stretch=stretch))
-            process_common_params(widget, params)
+                    r = make(row, lambda w: widget.add_widget(w,
+                                                              stretch=stretch))
+                    r = dict(row=r, stretch=stretch)
+                    res.append(r)
+
+            process_common_params('vbox', widget, params)
             pack(widget)
+            return ['vbox', params] + res
 
         # Sequence of separate top-level items
         def seq(params, cols, pack):
             def mypack(w):
                 w_top = self.app.make_window()
-                #w_top.cfg_expand(8, 8)
+                # w_top.cfg_expand(8, 8)
                 # Ask the size of the widget that wants to get packed
                 # and resize the top-level to fit
                 wd, ht = w.get_size()
@@ -483,46 +536,111 @@ class Desktop(Callback.Callbacks):
                 w_top.set_widget(w)
                 self.toplevels.append(w_top)
                 w_top.show()
+                process_common_params('top', w_top, params)
 
+            res = []
             for dct in cols:
                 if isinstance(dct, dict):
-                    stretch = dct.get('stretch', 0)
+                    stretch = dct.get('stretch', 0)  # noqa
                     col = dct.get('col', None)
                 else:
                     # assume a list defining the col
-                    stretch = 0
+                    stretch = 0  # noqa
                     col = dct
                 if col is not None:
-                    make(col, mypack)
+                    res.append(make(col, mypack))
 
             widget = Widgets.Label("Placeholder")
             pack(widget)
+            return ['seq', params] + res
 
         def make(constituents, pack):
             kind = constituents[0]
-            params = constituents[1]
+            params = Bunch.Bunch(constituents[1])
             if len(constituents) > 2:
                 rest = constituents[2:]
             else:
                 rest = []
 
             if kind == 'vpanel':
-                vpanel(params, rest, pack)
+                res = vpanel(params, rest, pack)
             elif kind == 'hpanel':
-                hpanel(params, rest, pack)
+                res = hpanel(params, rest, pack)
             elif kind == 'vbox':
-                vbox(params, rest, pack)
+                res = vbox(params, rest, pack)
             elif kind == 'hbox':
-                hbox(params, rest, pack)
+                res = hbox(params, rest, pack)
             elif kind == 'seq':
-                seq(params, rest, pack)
+                res = seq(params, rest, pack)
             elif kind in ('ws', 'mdi', 'widget'):
-                make_widget(kind, params, rest, pack)
+                res = make_widget(kind, params, rest, pack)
 
-        make(layout, lambda w: None)
+            return res
+
+        self.layout = make(layout, lambda w: None)
+        return self.layout
+
+    def record_sizes(self):
+        """Record sizes of all container widgets in the layout.
+        The sizes are recorded in the `params` mappings in the layout.
+        """
+        for rec in self.node.values():
+            w = rec.widget
+            wd, ht = w.get_size()
+
+            rec.params.update(dict(name=rec.name, width=wd, height=ht))
+
+            if rec.kind in ('hpanel', 'vpanel'):
+                sizes = w.get_sizes()
+                rec.params.update(dict(sizes=sizes))
+
+            if rec.kind in ('ws',):
+                # record ws type
+                # TODO: record positions of subwindows
+                rec.params.update(dict(wstype=rec.ws.wstype))
+
+            if rec.kind == 'top':
+                # record position for top-level widgets as well
+                x, y = w.get_pos()
+                if x is not None and y is not None:
+                    # we don't always get reliable information about position
+                    rec.params.update(dict(xpos=x, ypos=y))
+
+    def write_layout_conf(self, lo_file, layout=None):
+        if layout is None:
+            layout = self.layout
+
+        import pprint
+        self.record_sizes()
+
+        # write layout
+        with open(lo_file, 'w') as out_f:
+            pprint.pprint(layout, out_f)
+
+    def read_layout_conf(self, lo_file):
+        import ast
+
+        # read layout
+        with open(lo_file, 'r') as in_f:
+            buf = in_f.read()
+
+        layout = ast.literal_eval(buf)
+        return layout
+
+    def build_desktop(self, layout, lo_file=None, widget_dict=None):
+        if lo_file is not None:
+            alt_layout = layout
+            try:
+                layout = self.read_layout_conf(lo_file)
+
+            except Exception as e:
+                self.logger.error("Error reading saved layout: %s" % (str(e)))
+                layout = alt_layout
+
+        return self.make_desktop(layout, widget_dict=widget_dict)
 
 
-    ##### WORKSPACES #####
+# WORKSPACES #####
 
 class Workspace(Widgets.WidgetBase):
 
@@ -556,7 +674,6 @@ class Workspace(Widgets.WidgetBase):
             item = ws_menu.add_name("Close")
             item.add_callback('activated', self._close_menuitem_cb)
 
-
         self._set_wstype(wstype)
         self.vbox.add_widget(self.nb, stretch=1)
 
@@ -571,11 +688,11 @@ class Workspace(Widgets.WidgetBase):
             self.toolbar.add_widget(cbox)
 
             mdi_menu = self.toolbar.add_menu("MDI")
-            ## item = mdi_menu.add_name("Panes as Tabs", checkable=True)
-            ## item.add_callback('activated',
-            ##                   lambda w, tf: self.tabstoggle_cb(tf))
-            ## is_tabs = (self.nb.get_mode() == 'tabs')
-            ## item.set_state(is_tabs)
+            # item = mdi_menu.add_name("Panes as Tabs", checkable=True)
+            # item.add_callback('activated',
+            #                   lambda w, tf: self.tabstoggle_cb(tf))
+            # is_tabs = (self.nb.get_mode() == 'tabs')
+            # item.set_state(is_tabs)
 
             item = mdi_menu.add_name("Tile Panes")
             item.add_callback('activated',
@@ -587,14 +704,15 @@ class Workspace(Widgets.WidgetBase):
             self.mdi_menu = mdi_menu
             self._update_mdi_menu()
 
-        for name in ('page-switch', 'page-detach', 'ws-close'):
+        for name in ('page-switch', 'page-detach', 'page-close',
+                     'ws-close'):
             self.enable_callback(name)
 
     def _set_wstype(self, wstype):
         if wstype in ('tabs', 'nb', 'ws'):
             wstype = 'tabs'
             self.nb = Widgets.TabWidget(detachable=self.detachable,
-                                            group=self.group)
+                                        reorderable=True, group=self.group)
 
         elif wstype == 'mdi':
             self.nb = Widgets.MDIWidget(mode='mdi')
@@ -610,6 +728,8 @@ class Workspace(Widgets.WidgetBase):
             self.nb.add_callback('page-switch', self._switch_page_cb)
         if self.nb.has_callback('page-detach'):
             self.nb.add_callback('page-detach', self._detach_page_cb)
+        if self.nb.has_callback('page-close'):
+            self.nb.add_callback('page-close', self._close_page_cb)
 
         self.wstype = wstype
 
@@ -640,6 +760,9 @@ class Workspace(Widgets.WidgetBase):
     def _detach_page_cb(self, nb, child):
         self.make_callback('page-detach', child)
 
+    def _close_page_cb(self, nb, child):
+        self.make_callback('page-close', child)
+
     def _close_menuitem_cb(self, *args):
         self.make_callback('ws-close')
 
@@ -693,7 +816,10 @@ class Workspace(Widgets.WidgetBase):
             return False
 
         cur_idx = self.nb.get_index()
-        child = self.nb.index_to_widget(cur_idx)
+        try:
+            child = self.nb.index_to_widget(cur_idx)
+        except IndexError:
+            return
         _f(child)
 
     def to_next(self):
@@ -732,10 +858,17 @@ class SymmetricGridWidget(Widgets.GridBox):
 
         self.enable_callback('page-switch')
 
+    def _calc_dims(self, num_widgets):
+        rows = int(round(math.sqrt(num_widgets)))
+        cols = rows
+        if rows ** 2 < num_widgets:
+            cols += 1
+        return rows, cols
+
     def _relayout(self, widgets):
         # remove all the old widgets
         # TEMP: this call causes a recursive loop to happen
-        #self.remove_all()
+        # self.remove_all()
         children = list(self.get_children())
         for child in children:
             super(SymmetricGridWidget, self).remove(child)
@@ -743,10 +876,7 @@ class SymmetricGridWidget(Widgets.GridBox):
         # calculate number of rows and cols, try to maintain a square
         # TODO: take into account the window geometry
         num_widgets = len(widgets)
-        rows = int(round(math.sqrt(num_widgets)))
-        cols = rows
-        if rows**2 < num_widgets:
-            cols += 1
+        rows, cols = self._calc_dims(num_widgets)
 
         self.resize_grid(rows, cols)
 
@@ -756,37 +886,88 @@ class SymmetricGridWidget(Widgets.GridBox):
                 index = i*cols + j
                 if index < num_widgets:
                     child = widgets[index]
-                    super(SymmetricGridWidget, self).add_widget(child,
-                                                                i, j, stretch=1)
+                    super(SymmetricGridWidget, self).add_widget(child, i, j,
+                                                                stretch=1)
 
     def add_widget(self, child, title=''):
-        widgets = list(self.get_children())
-        widgets.append(child)
+        num_widgets = self.num_children()
+        o_rows, o_cols = self._calc_dims(num_widgets)
+
+        num_widgets += 1
+        rows, cols = self._calc_dims(num_widgets)
+
         # attach title to child
         child.extdata.tab_title = title
 
-        self._relayout(widgets)
+        if (o_rows, o_cols) == (rows, cols):
+            # size of matrix has not changed--
+            # we can be a little efficient here and skip rebuilding
+            j = num_widgets - ((rows - 1) * cols) - 1
+            super(SymmetricGridWidget, self).add_widget(child, rows-1, j,
+                                                        stretch=1)
+        else:
+            widgets = list(self.get_children())
+            widgets.append(child)
+            self._relayout(widgets)
 
     def remove(self, child, delete=False):
-        super(SymmetricGridWidget, self).remove(child)
 
         widgets = list(self.get_children())
+        try:
+            idx = widgets.index(child)
 
-        self._relayout(widgets)
+        except IndexError:
+            raise ValueError("child not in container")
+
+        o_rows, o_cols = self._calc_dims(len(widgets))
+
+        super(SymmetricGridWidget, self).remove(child)
+
+        num_widgets = self.num_children()
+        rows, cols = self._calc_dims(num_widgets)
+
+        # if size of matrix has not changed AND user only deleted from
+        # the end--we can be a little efficient here and skip rebuilding
+        if (child != widgets[-1]) or ((o_rows, o_cols) != (rows, cols)):
+            widgets = list(self.get_children())
+            self._relayout(widgets)
+
+        if idx < self.cur_index:
+            # deleted a widget lower than the current index:
+            # current index just shifts down
+            self.cur_index -= 1
+
+        elif idx == self.cur_index:
+            # deleted same widget as current index
+            self.cur_index = -1
+            num_children = self.num_children()
+            if 0 <= idx < num_children:
+                # we can keep same index
+                self.set_index(idx)
+            else:
+                # pick acceptable index
+                idx = max(0, min(idx - 1, num_children - 1))
+                if num_children > 0:
+                    self.set_index(idx)
 
     def get_index(self):
         return self.cur_index
 
     def set_index(self, idx):
         old_index = self.cur_index
-        if 0 <= idx < self.num_children():
+        num_children = self.num_children()
+        if 0 <= idx < num_children:
             self.cur_index = idx
             child = self.children[idx]
-            #child.focus()
+            # child.focus()
 
             if old_index != idx:
                 self.make_callback('page-switch', child)
 
+        else:
+            raise ValueError("index (%d) inconsistent with number of children (%d)" %
+                             (idx, num_children))
+
     def index_of(self, child):
         children = self.get_children()
         try:
@@ -798,4 +979,4 @@ class SymmetricGridWidget(Widgets.GridBox):
         children = self.get_children()
         return children[idx]
 
-#END
+# END
