@@ -61,20 +61,17 @@ class ImageViewMpl(ImageView.ImageViewBase):
 
         # NOTE: matplotlib manages it's Y coordinates by default with
         # the origin at the bottom (see base class)
-        self._originUpper = False
+        self.origin_upper = False
 
         self.img_fg = (1.0, 1.0, 1.0)
         self.img_bg = (0.5, 0.5, 0.5)
 
         self.in_axes = False
         # Matplotlib expects RGBA data for color images
-        self._rgb_order = 'RGBA'
+        self.rgb_order = 'RGBA'
 
         self.renderer = CanvasRenderer(self)
 
-        # cursors
-        self.cursor = {}
-
         # For timing events
         self._msg_timer = None
         self._defer_timer = None
@@ -111,13 +108,13 @@ class ImageViewMpl(ImageView.ImageViewBase):
         self._defer_timer = None
 
         if hasattr(figure.canvas, 'new_timer'):
-            self._msg_timer = Timer(0.0,
-                                    lambda timer: self.onscreen_message(None),
-                                    mplcanvas=figure.canvas)
+            self._msg_timer = Timer(mplcanvas=figure.canvas)
+            self._msg_timer.add_callback('expired',
+                                         lambda timer: self.onscreen_message(None))
 
-            self._defer_timer = Timer(0.0,
-                                      lambda timer: self.delayed_redraw(),
-                                      mplcanvas=figure.canvas)
+            self._defer_timer = Timer(mplcanvas=figure.canvas)
+            self._defer_timer.add_callback('expired',
+                                           lambda timer: self.delayed_redraw())
 
         canvas = figure.canvas
         if hasattr(canvas, 'viewer'):
@@ -139,9 +136,6 @@ class ImageViewMpl(ImageView.ImageViewBase):
     def get_widget(self):
         return self.figure.canvas
 
-    def get_rgb_order(self):
-        return self._rgb_order
-
     def calculate_aspect(self, shape, extent):
         dx = abs(extent[1] - extent[0]) / float(shape[1])
         dy = abs(extent[3] - extent[2]) / float(shape[0])
@@ -161,7 +155,7 @@ class ImageViewMpl(ImageView.ImageViewBase):
 
         # Grab the RGB array for the current image and place it in the
         # matplotlib figure axis
-        data = self.getwin_array(order=self._rgb_order)
+        data = self.getwin_array(order=self.rgb_order)
 
         dst_x = dst_y = 0
 
@@ -196,7 +190,7 @@ class ImageViewMpl(ImageView.ImageViewBase):
 
         # Grab the RGB array for the current image and place it in the
         # matplotlib figure axis
-        arr = self.getwin_array(order=self._rgb_order)
+        arr = self.getwin_array(order=self.rgb_order)
 
         # Get the data extents
         x0, y0 = 0, 0
@@ -292,18 +286,9 @@ class ImageViewMpl(ImageView.ImageViewBase):
     def set_cursor(self, cursor):
         pass
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
-
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
-
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
-
     def onscreen_message(self, text, delay=None, redraw=True):
         if self._msg_timer is not None:
-            self._msg_timer.cancel()
+            self._msg_timer.stop()
 
         self.set_onscreen_message(text, redraw=redraw)
 
@@ -315,7 +300,7 @@ class ImageViewMpl(ImageView.ImageViewBase):
             self.delayed_redraw()
             return
 
-        self._defer_timer.cancel()
+        self._defer_timer.stop()
         self._defer_timer.start(time_sec)
 
 
@@ -325,12 +310,6 @@ class ImageViewEvent(ImageViewMpl):
         ImageViewMpl.__init__(self, logger=logger, rgbmap=rgbmap,
                               settings=settings)
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
@@ -363,6 +342,16 @@ class ImageViewEvent(ImageViewMpl):
             'f10': 'f10',
             'f11': 'f11',
             'f12': 'f12',
+            'right': 'right',
+            'left': 'left',
+            'up': 'up',
+            'down': 'down',
+            'insert': 'insert',
+            'delete': 'delete',
+            'home': 'home',
+            'end': 'end',
+            'pageup': 'page_up',
+            'pagedown': 'page_down',
             }
 
         # Define cursors for pick and pan
@@ -404,7 +393,12 @@ class ImageViewEvent(ImageViewMpl):
         if keyname is None:
             return keyname
         try:
-            return self._keytbl[keyname.lower()]
+            key = keyname.lower()
+            if 'shift+' in key:
+                key = key.replace('shift+', '')
+            if 'ctrl+' in key:
+                key = key.replace('ctrl+', '')
+            return self._keytbl[key]
 
         except KeyError:
             return keyname
@@ -445,30 +439,30 @@ class ImageViewEvent(ImageViewMpl):
 
     def button_press_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button in (1, 2, 3):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button in (1, 2, 3):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, event):
         button = 0
         x, y = event.x, event.y
@@ -478,14 +472,14 @@ class ImageViewEvent(ImageViewMpl):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("motion event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
         self.logger.debug("motion event at DATA %dx%d" % (data_x, data_y))
 
         return self.make_ui_callback('motion', button, data_x, data_y)
 
     def scroll_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
 
         # Matplotlib only gives us the number of steps of the scroll,
         # positive for up and negative for down.  No horizontal scrolling.
@@ -500,8 +494,7 @@ class ImageViewEvent(ImageViewMpl):
         self.logger.debug("scroll deg=%f direction=%f" % (
             amount, direction))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('scroll', direction, amount,
                                   data_x, data_y)
