@@ -15,9 +15,7 @@ from ginga import ImageView, Mixins, Bindings
 import ginga.util.six as six
 from ginga.util.six.moves import map, zip
 from ginga.qtw.CanvasRenderQt import CanvasRenderer
-
-moduleHome = os.path.split(sys.modules[__name__].__file__)[0]
-icon_dir = os.path.abspath(os.path.join(moduleHome, '..', 'icons'))
+from ginga.util.paths import icondir
 
 
 class ImageViewQtError(ImageView.ImageViewError):
@@ -127,17 +125,18 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.imgwin.viewer = self
         self.pixmap = None
         # Qt expects 32bit BGRA data for color images
-        self._rgb_order = 'BGRA'
+        self.rgb_order = 'BGRA'
 
         self.renderer = CanvasRenderer(self)
 
-        self.msgtimer = Timer(0.0, lambda timer: self.onscreen_message_off())
-
-        # cursors
-        self.cursor = {}
+        self.msgtimer = Timer()
+        self.msgtimer.add_callback('expired',
+                                   lambda timer: self.onscreen_message_off())
 
         # For optomized redrawing
-        self._defer_task = Timer(0.0, lambda timer: self.delayed_redraw())
+        self._defer_task = Timer()
+        self._defer_task.add_callback('expired',
+                                      lambda timer: self.delayed_redraw())
 
     def get_widget(self):
         return self.imgwin
@@ -176,7 +175,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.logger.debug("drawing to pixmap")
 
         # Prepare array for rendering
-        arr = rgbobj.get_array(self._rgb_order)
+        arr = rgbobj.get_array(self.rgb_order)
         (height, width) = arr.shape[:2]
 
         return self._render_offscreen(self.pixmap, arr, dst_x, dst_y,
@@ -218,12 +217,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         qbuf.close()
         return ibuf
 
-    def get_rgb_image_as_bytes(self, format='png', quality=90):
-        ibuf = self.get_rgb_image_as_buffer(format=format, quality=quality)
-        return bytes(ibuf.getvalue())
-
-    def get_rgb_image_as_widget(self, output=None, format='png',
-                                quality=90):
+    def get_rgb_image_as_widget(self):
         imgwin_wd, imgwin_ht = self.get_window_size()
         qpix = self.pixmap.copy(0, 0,
                                 imgwin_wd, imgwin_ht)
@@ -233,23 +227,23 @@ class ImageViewQt(ImageView.ImageViewBase):
         qimg = self.get_rgb_image_as_widget()
         res = qimg.save(filepath, format=format, quality=quality)
 
-    def get_image_as_widget(self):
+    def get_plain_image_as_widget(self):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        arr = self.getwin_array(order=self._rgb_order)
+        arr = self.getwin_array(order=self.rgb_order)
         image = self._get_qimage(arr)
         return image
 
-    def save_image_as_file(self, filepath, format='png', quality=90):
+    def save_plain_image_as_file(self, filepath, format='png', quality=90):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        qimg = self.get_image_as_widget()
+        qimg = self.get_plain_image_as_widget()
         res = qimg.save(filepath, format=format, quality=quality)
 
     def reschedule_redraw(self, time_sec):
-        self._defer_task.cancel()
+        self._defer_task.stop()
         self._defer_task.start(time_sec)
 
     def update_image(self):
@@ -263,17 +257,16 @@ class ImageViewQt(ImageView.ImageViewBase):
                                   QtGui.QGraphicsScene.BackgroundLayer)
         else:
             self.imgwin.update()
-            #self.imgwin.show()
 
     def set_cursor(self, cursor):
         if self.imgwin is not None:
             self.imgwin.setCursor(cursor)
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
-
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
+    def make_cursor(self, iconpath, x, y):
+        image = QImage()
+        image.load(iconpath)
+        pm = QPixmap(image)
+        return QCursor(pm, x, y)
 
     def center_cursor(self):
         if self.imgwin is None:
@@ -297,11 +290,8 @@ class ImageViewQt(ImageView.ImageViewBase):
         cursor = self.imgwin.cursor()
         cursor.setPos(s_pt)
 
-    def get_rgb_order(self):
-        return self._rgb_order
-
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
+    def make_timer(self):
+        return Timer()
 
     def _get_qimage(self, bgra):
         h, w, channels = bgra.shape
@@ -318,14 +308,11 @@ class ImageViewQt(ImageView.ImageViewBase):
         return clr
 
     def onscreen_message(self, text, delay=None, redraw=True):
-        self.msgtimer.cancel()
+        self.msgtimer.stop()
         self.set_onscreen_message(text, redraw=redraw)
         if delay is not None:
             self.msgtimer.start(delay)
 
-    def onscreen_message_off(self):
-        return self.onscreen_message(None)
-
 
 class RenderMixin(object):
 
@@ -399,21 +386,11 @@ class RenderWidgetZoom(RenderMixin, RenderWidget):
 class RenderGraphicsViewZoom(RenderMixin, RenderGraphicsView):
     pass
 
-class ImageViewEvent(ImageViewQt):
 
-    def __init__(self, logger=None, rgbmap=None, settings=None, render=None):
-        ImageViewQt.__init__(self, logger=logger, rgbmap=rgbmap,
-                             settings=settings, render=render)
+class QtEventMixin(object):
 
-        # replace the widget our parent provided
-        if self.wtype == 'scene':
-            imgwin = RenderGraphicsViewZoom()
-            imgwin.setScene(self.scene)
-        else:
-            imgwin = RenderWidgetZoom()
-
-        imgwin.viewer = self
-        self.imgwin = imgwin
+    def __init__(self):
+        imgwin = self.imgwin
         imgwin.setFocusPolicy(QtCore.Qt.FocusPolicy(
                               QtCore.Qt.TabFocus |
                               QtCore.Qt.ClickFocus |
@@ -431,20 +408,14 @@ class ImageViewEvent(ImageViewQt):
         #imgwin.grabGesture(QtCore.Qt.TapGesture)
         #imgwin.grabGesture(QtCore.Qt.TapAndHoldGesture)
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
                                ('pick', 'thinCrossCursor.png')):
-            path = os.path.join(icon_dir, filename)
-            cur = make_cursor(path, 8, 8)
+            path = os.path.join(icondir, filename)
+            cur = self.make_cursor(path, 8, 8)
             self.define_cursor(curname, cur)
 
         # @$%&^(_)*&^ qt!!
@@ -469,6 +440,9 @@ class ImageViewEvent(ImageViewQt):
                      'pinch', 'pan', 'swipe', 'tap'):
             self.enable_callback(name)
 
+    def set_enter_focus(self, tf):
+        self.enter_focus = tf
+
     def transkey(self, keycode, keyname):
         self.logger.debug("keycode=%d keyname='%s'" % (
             keycode, keyname))
@@ -520,9 +494,6 @@ class ImageViewEvent(ImageViewQt):
     def get_keyTable(self):
         return self._keytbl
 
-    def set_enter_focus(self, tf):
-        self.enter_focus = tf
-
     def map_event(self, widget, event):
         rect = widget.geometry()
         x1, y1, x2, y2 = rect.getCoords()
@@ -561,6 +532,7 @@ class ImageViewEvent(ImageViewQt):
     def button_press_event(self, widget, event):
         buttons = event.buttons()
         x, y = event.x(), event.y()
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         if buttons & QtCore.Qt.LeftButton:
@@ -571,13 +543,15 @@ class ImageViewEvent(ImageViewQt):
             button |= 0x4
         self.logger.debug("button down event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, widget, event):
         # note: for mouseRelease this needs to be button(), not buttons()!
         buttons = event.button()
         x, y = event.x(), event.y()
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         if buttons & QtCore.Qt.LeftButton:
@@ -587,16 +561,21 @@ class ImageViewEvent(ImageViewQt):
         if buttons & QtCore.Qt.RightButton:
             button |= 0x4
 
-        data_x, data_y = self.get_data_xy(x, y)
+        data_x, data_y = self.check_cursor_location()
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, widget, event):
+
+        if self.is_redraw_pending():
+            # NOTE: this hack works around a bug in Qt5 (ver < 5.6.2)
+            # where mouse motion events are not compressed properly,
+            # causing many events to build up in the queue and slowing
+            # down the viewer due to each one forcing a redraw.
+            # This test tells us there is a deferred redraw waiting;
+            # discard any motion events until the redraw happens.
+            return True
+
         buttons = event.buttons()
         x, y = event.x(), event.y()
         self.last_win_x, self.last_win_y = x, y
@@ -609,8 +588,7 @@ class ImageViewEvent(ImageViewQt):
         if buttons & QtCore.Qt.RightButton:
             button |= 0x4
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('motion', button, data_x, data_y)
 
@@ -618,14 +596,24 @@ class ImageViewEvent(ImageViewQt):
         x, y = event.x(), event.y()
         self.last_win_x, self.last_win_y = x, y
 
-        numDegrees, direction = get_scroll_info(event)
+        num_degrees, direction = get_scroll_info(event)
         self.logger.debug("scroll deg={} direction={}".format(
-            numDegrees, direction))
+            num_degrees, direction))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        # NOTE: for future use in distinguishing mouse wheel vs.
+        # trackpad events
+        src = 'wheel'
+        if hasattr(event, 'source'):
+            # Qt5 only, it seems
+            _src = event.source()
+            if _src == QtCore.Qt.MouseEventNotSynthesized:
+                src = 'wheel'
+            else:
+                src = 'trackpad'
 
-        return self.make_ui_callback('scroll', direction, numDegrees,
+        data_x, data_y = self.check_cursor_location()
+
+        return self.make_ui_callback('scroll', direction, num_degrees,
                                      data_x, data_y)
 
     def gesture_event(self, widget, event):
@@ -746,6 +734,26 @@ class ImageViewEvent(ImageViewQt):
         #event.acceptProposedAction()
         self.make_ui_callback('drag-drop', data)
 
+
+class ImageViewEvent(QtEventMixin, ImageViewQt):
+
+    def __init__(self, logger=None, rgbmap=None, settings=None, render=None):
+        ImageViewQt.__init__(self, logger=logger, rgbmap=rgbmap,
+                             settings=settings, render=render)
+
+        # replace the widget our parent provided
+        if self.wtype == 'scene':
+            imgwin = RenderGraphicsViewZoom()
+            imgwin.setScene(self.scene)
+        else:
+            imgwin = RenderWidgetZoom()
+
+        imgwin.viewer = self
+        self.imgwin = imgwin
+
+        QtEventMixin.__init__(self)
+
+
 class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
 
     # class variables for binding map and bindings can be set
@@ -767,7 +775,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 rgbmap=rgbmap, render=render)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_setActive(True)
+        self.ui_set_active(True)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
@@ -824,7 +832,12 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         self.scroll_bars(horizontal='on', vertical='on')
 
         self._adjusting = False
+        self._scrolling = False
         self.pad = 20
+        self.range_h = 10000
+        self.range_v = 10000
+        self.upper_h = self.range_h
+        self.upper_v = self.range_v
 
         # reparent the viewer widget
         vp = self.viewport()
@@ -833,10 +846,10 @@ class ScrolledView(QtGui.QAbstractScrollArea):
 
         hsb = self.horizontalScrollBar()
         hsb.setTracking(True)
-        hsb.setRange(0, 100)
+        hsb.setRange(0, self.upper_h)
         hsb.setSingleStep(1)
         vsb = self.verticalScrollBar()
-        vsb.setRange(0, 100)
+        vsb.setRange(0, self.upper_v)
         vsb.setSingleStep(1)
         vsb.setTracking(True)
 
@@ -862,6 +875,9 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         """Calculate and set the scrollbar handles from the pan and
         zoom positions.
         """
+        if self._scrolling:
+            return
+
         # flag that suppresses a cyclical callback
         self._adjusting = True
         try:
@@ -873,11 +889,22 @@ class ScrolledView(QtGui.QAbstractScrollArea):
             hsb = self.horizontalScrollBar()
             vsb = self.verticalScrollBar()
 
-            hsb.setPageStep(int(round(res.thm_pct_x * 100)))
-            vsb.setPageStep(int(round(res.thm_pct_y * 100)))
+            page_h, page_v = (int(round(res.thm_pct_x * self.range_h)),
+                              int(round(res.thm_pct_y * self.range_v)))
+            hsb.setPageStep(page_h)
+            vsb.setPageStep(page_v)
+
+            upper_h = max(1, self.range_h - page_h)
+            upper_v = max(1, self.range_v - page_v)
+            hsb.setRange(0, upper_h)
+            vsb.setRange(0, upper_v)
+            self.upper_h, self.upper_v = upper_h, upper_v
+
+            val_h, val_v = (int(round(res.pan_pct_x * self.upper_h)),
+                            int(round((1.0 - res.pan_pct_y) * self.upper_v)))
+            hsb.setValue(val_h)
+            vsb.setValue(val_v)
 
-            hsb.setValue(int(round(res.pan_pct_x * 100)))
-            vsb.setValue(int(round((1.0 - res.pan_pct_y) * 100)))
         finally:
             self._adjusting = False
 
@@ -888,14 +915,30 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         if self._adjusting:
             return
 
-        hsb = self.horizontalScrollBar()
-        vsb = self.verticalScrollBar()
-        pct_x = hsb.value() / 100.0
-        # invert Y pct because of orientation of scrollbar
-        pct_y = 1.0 - (vsb.value() / 100.0)
+        self._scrolling = True
+        try:
+            bd = self.viewer.get_bindings()
+            res = bd.calc_pan_pct(self.viewer, pad=self.pad)
+            if res is None:
+                return
+            pct_x, pct_y = res.pan_pct_x, res.pan_pct_y
 
-        bd = self.viewer.get_bindings()
-        bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
+            # Only adjust pan setting for axes that have changed
+            if dx != 0:
+                hsb = self.horizontalScrollBar()
+                pos_x = float(hsb.value())
+                pct_x = pos_x / float(self.upper_h)
+            if dy != 0:
+                vsb = self.verticalScrollBar()
+                pos_y = float(vsb.value())
+                # invert Y pct because of orientation of scrollbar
+                pct_y = 1.0 - (pos_y / float(self.upper_v))
+
+            bd = self.viewer.get_bindings()
+            bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
+
+        finally:
+            self._scrolling = False
 
     def scroll_bars(self, horizontal='on', vertical='on'):
         if horizontal == 'on':
@@ -917,11 +960,4 @@ class ScrolledView(QtGui.QAbstractScrollArea):
             raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (vertical))
 
 
-def make_cursor(iconpath, x, y):
-    image = QImage()
-    image.load(iconpath)
-    pm = QPixmap(image)
-    return QCursor(pm, x, y)
-
-
 #END
